import pandas as pd
import requests
from bs4 import BeautifulSoup

# from datasets import load_dataset
from fastapi import APIRouter

# from sklearn.metrics import accuracy_score
# from sklearn.model_selection import train_test_split
# from transformers import (
#     AutoModelForQuestionAnswering,
#     AutoTokenizer,
#     DefaultDataCollator,
#     Trainer,
#     TrainingArguments,
#     pipeline,
# )

# router = APIRouter()


@router.get("/classify_demo2")
async def classifyDemo2():
    print("文本分类训练2开始")
    try:
        # 假设已有带标签的训练集
        training_data = [
            {
                "title": "Tech Article",
                "content": "This is a tech-related article.",
                "category": "Tech",
            },
            {
                "title": "Sports News",
                "content": "Latest sports news.",
                "category": "Sports",
            },
            {
                "title": "General Article",
                "content": "A general article on various topics.",
                "category": "Other",
            },
            # ...
        ]

        # 转为DataFrame
        df_train = pd.DataFrame(training_data)

        # 划分训练集和测试集
        df_train, df_test = train_test_split(df_train, test_size=0.2, random_state=42)

        # 使用transformers的pipeline进行分类
        classifier = pipeline(
            "text-classification",
            model="bert-base-uncased",
            tokenizer="bert-base-uncased",
        )
        classifier.train(df_train, max_epochs=3)

        # 在测试集上进行预测
        predictions = classifier.predict(df_test["content"])

        # 评估分类器性能
        accuracy = accuracy_score(df_test["category"], predictions)
        print(f"Accuracy: {accuracy}")

        # 使用训练好的模型进行新文章分类
        new_article = {
            "title": "New Article",
            "content": "This is a new article on technology.",
            "category": None,
        }
        predicted_category = classifier.predict([new_article["content"]])[0]

        print(f"Predicted Category: {predicted_category}")

        print("文本分类训练结束（只是一个例子，内部没有正式启动训练的过程）")
        return {"message": "success"}
    except Exception as e:
        print(f"classifyDemo2 出错 {e}")
        return {"message": "error"}


@router.get("/classify_demo3")
async def classifyDemo3():
    try:
        # 参考文章： https://developer.aliyun.com/article/1111665
        # RSS feed link and their categories. Will iterate them one by one.
        # Have mentioned only one feed for demo purposes
        timesofindia = {
            "world": "http://timesofindia.indiatimes.com/rssfeeds/296589292.cms"
        }
        for category, rsslink in timesofindia.items():
            print(f"Processing for category: {category}. \nRSS link: {rsslink}")
            # get the webpage URL and read the html
            rssdata = requests.get(rsslink)
            # print(rssdata.content)
            soup = BeautifulSoup(rssdata.content)
            print(soup.prettify())

        # get all news items. It has title, description, link, guid, pubdate for each news items.
        # Lets call this items and we will iterate thru it
        allitems = soup.find_all("item")  # print one news item/healine to check
        for item in range(len(allitems)):
            print("Processing news-item #:", item)
            title = allitems[item].title.text
            link = allitems[item].guid.text
            pubdate = allitems[item].pubdate.text
            print("TITLE:", title)
            print("LINK:", link)
            print("PUBDATE:", pubdate)

        # 提取文章
        # Function to fetch each news link to get news essay
        def fetch_news_text(link):
            # read the html webpage and parse it
            soup = BeautifulSoup(requests.get(link).content, "html.parser")
            # fetch the news article text box
            # these are with element <div class="_3WlLe clearfix">
            text_box = soup.find_all("div", attrs={"class": "_3WlLe clearfix"})
            # extract text and combine
            news_text = str(". ".join(t.text.strip() for t in text_box))
            return news_text  # using the above function, process text

        news_articles = [
            {
                "Feed": "timesofindia",
                "Category": category,
                "Headline": allitems[item].title.text,
                "Link": allitems[item].guid.text,
                "Pubdate": allitems[item].pubdate.text,
                "NewsText": fetch_news_text(allitems[item].guid.text),
            }
            for item in range(len(allitems))
        ]
        news_articles = pd.DataFrame(news_articles)
        news_articles.head(3)

        # 为了进行文本清理，我使用了文本的预处理，这些步骤是删除HTML标记，特殊字符，数字，标点符号，停用词，处理重音字符，扩展收缩，词干和词形等。
        # 在这里，我将这些预处理步骤放到一个函数中，该函数将返回干净且标准化的语料库。
        # test normalize cleanup on one article
        # clean_sentences = normalize_corpus([news_articles['NewsText'][0]])
        clean_sentences = normalize_corpus(news_articles["NewsText"])
        print("clean_sentences", clean_sentences)
        return {"message": "success"}
    except Exception as e:
        print(f"classifyDemo3 出错 {e}")
        return {"message": "error"}


@router.get("/classify_demo4")
async def classifyDemo4():
    """
    本地训练，训练完成后自动将模型上传到 huggingface
    "# 来自官方例子： https://huggingface.co/docs/transformers/v4.37.2/en/tasks/question_answering\n",
    "# 简单的问答系统，使用官方数据集训练，训练完成后发布到自己的 huggingface仓库。"
    """
    print("classify_demo4 开始")
    try:
        # 登录
        print("登录到hugging face")
        # hf.notebook_login()
        squad = load_dataset("squad", split="train[:5000]")
        squad = squad.train_test_split(test_size=0.2)
        squad["train"][0]

        # 步骤2：
        tokenizer = AutoTokenizer.from_pretrained("distilbert-base-uncased")

        def preprocess_function(examples):
            questions = [q.strip() for q in examples["question"]]
            inputs = tokenizer(
                questions,
                examples["context"],
                max_length=384,
                truncation="only_second",
                return_offsets_mapping=True,
                padding="max_length",
            )

            offset_mapping = inputs.pop("offset_mapping")
            answers = examples["answers"]
            start_positions = []
            end_positions = []

            for i, offset in enumerate(offset_mapping):
                answer = answers[i]
                start_char = answer["answer_start"][0]
                end_char = answer["answer_start"][0] + len(answer["text"][0])
                sequence_ids = inputs.sequence_ids(i)

                # Find the start and end of the context
                idx = 0
                while sequence_ids[idx] != 1:
                    idx += 1
                context_start = idx
                while sequence_ids[idx] == 1:
                    idx += 1
                context_end = idx - 1

                # If the answer is not fully inside the context, label it (0, 0)
                if (
                    offset[context_start][0] > end_char
                    or offset[context_end][1] < start_char
                ):
                    start_positions.append(0)
                    end_positions.append(0)
                else:
                    # Otherwise it's the start and end token positions
                    idx = context_start
                    while idx <= context_end and offset[idx][0] <= start_char:
                        idx += 1
                    start_positions.append(idx - 1)

                    idx = context_end
                    while idx >= context_start and offset[idx][1] >= end_char:
                        idx -= 1
                    end_positions.append(idx + 1)

            inputs["start_positions"] = start_positions
            inputs["end_positions"] = end_positions
            return inputs

        tokenized_squad = squad.map(
            preprocess_function,
            batched=True,
            remove_columns=squad["train"].column_names,
        )
        data_collator = DefaultDataCollator()
        print("数据准备就绪，开始训练")
        # from transformers import AutoModelForQuestionAnswering, TrainingArguments, Trainer
        model = AutoModelForQuestionAnswering.from_pretrained("distilbert-base-uncased")
        training_args = TrainingArguments(
            output_dir="my_awesome_qa_model",
            evaluation_strategy="epoch",
            learning_rate=2e-5,
            per_device_train_batch_size=16,
            per_device_eval_batch_size=16,
            num_train_epochs=3,
            weight_decay=0.01,
            push_to_hub=True,
        )

        trainer = Trainer(
            model=model,
            args=training_args,
            train_dataset=tokenized_squad["train"],
            eval_dataset=tokenized_squad["test"],
            tokenizer=tokenizer,
            data_collator=data_collator,
        )

        trainer.train()
        # 发布模型
        trainer.push_to_hub()
        return {"message": "success"}
    except Exception as e:
        print(f"classifyDemo4 出错 {e}")
        return {"message": str(e)}



import asyncio
import json

# import os
from contextvars import ContextVar

# from functools import lru_cache
from typing import Type

import httpx
import orjson
import structlog
from attr import make_class

# from crewai import LLM
from langchain_core.prompts import ChatPromptTemplate, PromptTemplate
from langchain_core.tools import StructuredTool
from langchain_core.utils.function_calling import convert_to_openai_function
from langchain_openai import ChatOpenAI
from langgraph.checkpoint.postgres.aio import AsyncPostgresSaver, BasePostgresSaver
from lazify import LazyProxy
from mtmaisdk import Context as HatchetContext
from psycopg_pool import AsyncConnectionPool
from pydantic import BaseModel

# from mtmai.agents import utils
from mtmai.agents.graphutils import ensure_valid_llm_response_v2
from mtmai.core.config import settings
from mtmai.mtlibs.httpx_transport import LoggingTransport

LOG = structlog.get_logger()

context_var: ContextVar["AgentContext"] = ContextVar("mtmai")


class AgentContext:
    tenant_id: str | None
    user_id: str | None
    thread_id: str | None
    cache_checkpointer: BasePostgresSaver | None

    def __init__(
        self,
        *,
        thread_id: str | None = None,
        user_id: str | None = None,
        tenant_id: str | None = None,
    ):
        self.tenant_id = tenant_id
        self.user_id = user_id
        self.thread_id = thread_id
        # self.httpx_session: httpx.Client = None
        # self.db: Engine = db_engine
        # self.session: Session = Session(db_engine)
        # embedding = get_default_embeddings()

        # self.vectorstore = MtmDocStore(session=Session(db_engine), embedding=embedding)
        # self.kvstore = MtmKvStore(db_engine)

        # self.graph_config = get_graph_config()
        self._mq = None
        # self.memMq = MemoryMQ()
        self._thread_id = thread_id

    def set_hatch_context(self, ctx: HatchetContext):
        self.hatchet_ctx = ctx

    def log(self, lineformat, **kw_args):
        if self.hatchet_ctx:
            self.hatchet_ctx.log(
                lineformat,
                kw_args,
            )
        else:
            LOG.info(lineformat, kw_args)

    def retrive_graph_config(self):
        return self.graph_config

    # def load_doc(self):
    #     return self.vectorstore

    # async def get_crawai_llm(self, name: str = "chat"):
    #     # llm_item = await self.get_llm_config(name)
    #     # def my_custom_logging_fn(model_call_dict):
    #     #     print(f"model call details=======================: {model_call_dict}")

    #     return LLM(
    #         model="openai/llama3.1-70b",
    #         # temperature=llm_item.temperature or None,
    #         base_url="https://llama3-1-70b.lepton.run/api/v1/",
    #         api_key="iByWYsIUIBe6qRYBswhLVPRyiVKkYb8r",
    #         num_retries=5,
    #         # logger_fn=my_custom_logging_fn,
    # )

    async def get_llm_openai(self, llm_config_name: str):
        # llm_item = await self.get_llm_config(llm_config_name)

        # base_url = llm_item.base_url
        # model = llm_item.model

        # all_llm_providers_prefix = ["together_ai/", "groq/"]
        # 		Model:      "openai/llama3.1-70b",
        # ApiKey:     mtutils.Ptr("CzEDAeske7RipomNe3KLLtqvu820Ewfp"),
        # BaseUrl:    mtutils.Ptr("https://llama3-1-70b.lepton.run/api/v1/"),

        # for prefix in all_llm_providers_prefix:
        #     if model.startswith(prefix):
        #         model = model[len(prefix) :]
        #         break
        return ChatOpenAI(
            base_url="https://llama3-1-70b.lepton.run/api/v1/",
            api_key="CzEDAeske7RipomNe3KLLtqvu820Ewfp",  # type: ignore
            model="openai/llama3.1-70b",
            temperature=0,
            # max_tokens=None,
            # 使用自定义 httpx 客户端 方便日志查看
            http_client=httpx.Client(transport=LoggingTransport()),  # type: ignore
            http_async_client=httpx.AsyncClient(transport=LoggingTransport()),
        )

    async def ainvoke_model(
        self,
        tpl: ChatPromptTemplate,
        inputs: dict | BaseModel | None,
        *,
        tools: list[StructuredTool | dict] | None = None,
        structured_output: BaseModel | None = None,
        llm_config_name: str = "chat",
        max_retries: int = 5,
        sleep_time: int = 3,
    ):
        llm_inst = await self.get_llm_openai(llm_config_name)

        if tools is not None and len(tools) > 0:
            formatted_tools = [
                convert_to_openai_function(tool, strict=True) for tool in tools
            ]
            all_tool_names = [t["name"] for t in formatted_tools]
            all_tool_names_str = ", ".join(all_tool_names)
            if tools and llm_inst.llm_type == "llama3.1":
                # for openai_fun in openai_functions:
                #     tool_call_prompts.append(f"""\nUse the function '{openai_fun["name"]}' to '{openai_fun["description"]}':\n{json.dumps(openai_fun)}\n""")
                # llama3.1 模型工具调用专用提示词，确保工具调用的准确性和一致性
                toolPrompt = f"""
    all tools: {all_tool_names_str}
    [IMPORTANT] When calling a function, adhere strictly to the following guidelines:
    1. Use the exact OpenAI ChatGPT function calling format.
    2. Function calls must be in this format: {{\"name\": \"function_name\", \"arguments\": {{\"arg1\": \"value1\", \"arg2\": \"value2\"}}}}
    3. Only call one function at a time.
    4. Do not include any additional text with the function call.
    5. If no function call is needed, respond normally without mentioning functions.
    6. Only use functions from the provided list of tools.
    7. Function names must consist solely of lowercase letters (a-z), numbers (0-9), and underscores (_).
    8. Ensure all required parameters for the function are included.
    9. Double-check that the function name and all parameter names exactly match those provided in the function description.

    If you're unsure about making a function call, respond to the user's query using your general knowledge instead.
    """
                if "additional_instructions" in tpl.partial_variables:
                    tpl = tpl.partial(additional_instructions=toolPrompt)
                else:
                    tpl.messages.append(
                        ChatPromptTemplate.from_messages([("system", toolPrompt)])
                    )

        if isinstance(inputs, BaseModel):
            inputs = inputs.model_dump()
        messages = await tpl.ainvoke(inputs)
        llm_chain = llm_inst
        if structured_output:
            llm_chain = llm_chain.with_structured_output(
                structured_output, include_raw=True
            )
        if tools:
            llm_chain = llm_chain.bind_tools(tools)
        llm_chain = llm_chain.with_retry(stop_after_attempt=5)

        message_to_post = messages.to_messages()

        for attempt in range(max_retries):
            try:
                invoke_result = await ensure_valid_llm_response_v2(
                    llm_chain, message_to_post
                )
                if isinstance(invoke_result, dict) and "raw" in invoke_result:
                    ai_msg = invoke_result["raw"]
                else:
                    ai_msg = invoke_result
                if tools:
                    ai_msg = utils.fix_tool_calls(ai_msg)

                # 函数名必须是 tools 内，否则必定是不正确的调用，自动重试
                tcs = ai_msg.tool_calls
                if tcs:
                    for tc in tcs:
                        if tc["name"] not in all_tool_names:
                            raise ValueError(
                                f"函数名 {tc['name']} 必须是 tools 内，否则必定是错误"
                            )
                return ai_msg
            except Exception as e:
                if attempt < max_retries - 1:
                    LOG.warning(
                        f"Attempt {attempt + 1} failed. Retrying in 5 seconds..."
                    )
                    await asyncio.sleep(sleep_time)
                else:
                    LOG.error(f"All {max_retries} attempts failed.")
                    raise e

    async def ainvoke_model_with_structured_output(
        self,
        tpl: PromptTemplate,
        inputs: dict | BaseModel | None,
        *,
        tools: list[StructuredTool | dict] = None,
        structured_output: BaseModel,
        llm_config_name: str = "chat",
        max_retries: int = 5,
        sleep_time: int = 3,
    ):
        # llm_item = await self.get_llm_config(llm_config_name)
        llm_inst = await self.get_llm_openai(llm_config_name)

        if tools is not None and len(tools) > 0:
            formatted_tools = [
                convert_to_openai_function(tool, strict=True) for tool in tools
            ]
            all_tool_names = [t["name"] for t in formatted_tools]
            all_tool_names_str = ", ".join(all_tool_names)
            if tools and llm_inst.llm_type == "llama3.1":
                # for openai_fun in openai_functions:
                #     tool_call_prompts.append(f"""\nUse the function '{openai_fun["name"]}' to '{openai_fun["description"]}':\n{json.dumps(openai_fun)}\n""")
                # llama3.1 模型工具调用专用提示词，确保工具调用的准确性和一致性
                toolPrompt = f"""
    all tools: {all_tool_names_str}
    [IMPORTANT] When calling a function, adhere strictly to the following guidelines:
    1. Use the exact OpenAI ChatGPT function calling format.
    2. Function calls must be in this format: {{\"name\": \"function_name\", \"arguments\": {{\"arg1\": \"value1\", \"arg2\": \"value2\"}}}}
    3. Only call one function at a time.
    4. Do not include any additional text with the function call.
    5. If no function call is needed, respond normally without mentioning functions.
    6. Only use functions from the provided list of tools.
    7. Function names must consist solely of lowercase letters (a-z), numbers (0-9), and underscores (_).
    8. Ensure all required parameters for the function are included.
    9. Double-check that the function name and all parameter names exactly match those provided in the function description.

    If you're unsure about making a function call, respond to the user's query using your general knowledge instead.
    """
                if "additional_instructions" in tpl.input_variables:
                    tpl = tpl.partial(additional_instructions=toolPrompt)
                else:
                    tpl.messages.append(
                        ChatPromptTemplate.from_messages([("system", toolPrompt)])
                    )

        if isinstance(inputs, BaseModel):
            inputs = inputs.model_dump()
        messages = await tpl.ainvoke(inputs)
        llm_chain = llm_inst
        if structured_output:
            llm_chain = llm_chain.with_structured_output(
                structured_output, include_raw=True
            )
        if tools:
            llm_chain = llm_chain.bind_tools(tools)
        llm_chain = llm_chain.with_retry(stop_after_attempt=5)

        message_to_post = messages.to_messages()

        for attempt in range(max_retries):
            try:
                invoke_result = await ensure_valid_llm_response_v2(
                    llm_chain, message_to_post
                )
                if isinstance(invoke_result, dict) and "raw" in invoke_result:
                    ai_msg = invoke_result["raw"]
                else:
                    ai_msg = invoke_result
                if tools:
                    ai_msg = utils.fix_tool_calls(ai_msg)

                # 函数名必须是 tools 内，否则必定是不正确的调用，自动重试
                # tcs = ai_msg.tool_calls
                # if tcs:
                #     for tc in tcs:
                #         if tc.name not in all_tool_names:
                #             raise ValueError(
                #                 f"函数名 {tc['name']} 必须是 tools 内，否则必定是错误"
                # )
                return invoke_result
            except Exception as e:
                if attempt < max_retries - 1:
                    LOG.warning(
                        f"Attempt {attempt + 1} failed. Retrying in 5 seconds..."
                    )
                    await asyncio.sleep(sleep_time)
                else:
                    LOG.error(f"All {max_retries} attempts failed.")
                    raise e

    async def stream_messages(self, tpl: ChatPromptTemplate, messages: list[any]):
        messages2 = await tpl.ainvoke({"messages": messages})
        # config = {"configurable": {"thread_id": "abc123"}}
        LOG.info(f"stream_messages: {messages2}")
        llm_inst = await self.get_llm_openai("chat")
        async for chunk in llm_inst.astream(
            messages2,
            # config,
        ):
            if chunk.content:
                yield chunk.content

    def load_json_response(
        self, ai_json_resonse_text: str, model_class: Type[BaseModel]
    ) -> Type[BaseModel]:
        repaired_json = self.repair_json(ai_json_resonse_text)
        try:
            loaded_data = orjson.loads(repaired_json)
            return make_class(**loaded_data)
        except Exception as e:
            LOG.error(f"Error parsing JSON: {str(e)}")
            raise ValueError(
                f"Failed to parse JSON and create {model_class.__name__} instance"
            ) from e

    async def get_db_pool(self):
        connection_kwargs = {
            "autocommit": True,
            "prepare_threshold": 0,
        }
        if not settings.MTMAI_DATABASE_URL:
            raise ValueError("MTMAI_DATABASE_URL is not set")
        pool = AsyncConnectionPool(
            conninfo=settings.MTMAI_DATABASE_URL,
            max_size=20,
            kwargs=connection_kwargs,
        )
        await pool.open()
        return pool

    async def checkpointer(self):
        pool = await self.get_db_pool()
        # if not settings.MTMAI_DATABASE_URL:
        #     raise ValueError("MTMAI_DATABASE_URL is not set")
        # db_str = settings.MTMAI_DATABASE_URL
        # checkpointer = AsyncPostgresSaver.from_conn_string(db_str)
        # aa= await AsyncPostgresSaver.from_conn_string(
        #         settings.MTMAI_DATABASE_URL
        #     )

        checkpointer = AsyncPostgresSaver(pool)  # type: ignore
        # if not self.cache_checkpointer:
        #     self.cache_checkpointer = AsyncPostgresSaver.from_conn_string(
        #         settings.MTMAI_DATABASE_URL
        #     )
        return checkpointer

    async def get_graph_by_name(self, name: str):
        if name == "storm":
            from mtmai.agents.storm.storm import StormGraph

            return StormGraph()

        return None

    async def emit(self, evt_name: str, data: dict):
        """发送事件"""
        json1 = json.dumps(data)
        self.hatchet_ctx.put_stream(f"0:{json1}\n")


def init_mtmai_context(hatchetCtx: HatchetContext) -> AgentContext:
    tenant_id = None
    user_id = None
    additional_metadata = hatchetCtx.action.additional_metadata
    if additional_metadata:
        tenant_id = additional_metadata.get("tenantId")
        user_id = additional_metadata.get("userId")
    agent_ctx = AgentContext(tenant_id=tenant_id, user_id=user_id)
    agent_ctx.hatchet_ctx = hatchetCtx
    context_var.set(agent_ctx)
    return agent_ctx


def get_mtmai_context() -> AgentContext:
    try:
        return context_var.get()
    except LookupError:
        raise RuntimeError("mtmai_context  error")


mtmai_context: AgentContext = LazyProxy(get_mtmai_context, enable_cache=False)  # type: ignore

mtmai_context: AgentContext = LazyProxy(get_mtmai_context, enable_cache=False)  # type: ignore
mtmai_context: AgentContext = LazyProxy(get_mtmai_context, enable_cache=False)  # type: ignore
mtmai_context: AgentContext = LazyProxy(get_mtmai_context, enable_cache=False)  # type: ignore
mtmai_context: AgentContext = LazyProxy(get_mtmai_context, enable_cache=False)  # type: ignore



# coding: utf-8

"""
    Mtmai API

    The Mtmai API

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from mtmai.gomtmclients.rest.models.tool_config import ToolConfig
from typing import Optional, Set
from typing_extensions import Self

class MultimodalWebSurferAgentConfig(BaseModel):
    """
    MultimodalWebSurferAgentConfig
    """ # noqa: E501
    component_type: StrictStr
    version: Optional[StrictStr] = None
    description: StrictStr
    name: StrictStr
    agent_type: StrictStr
    system_message: Optional[StrictStr] = None
    model_client: Optional[Any]
    tools: List[ToolConfig]
    __properties: ClassVar[List[str]] = ["component_type", "version", "description", "name", "agent_type", "system_message", "model_client", "tools"]

    @field_validator('agent_type')
    def agent_type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['MultimodalWebSurfer']):
            raise ValueError("must be one of enum values ('MultimodalWebSurfer')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of MultimodalWebSurferAgentConfig from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in tools (list)
        _items = []
        if self.tools:
            for _item_tools in self.tools:
                if _item_tools:
                    _items.append(_item_tools.to_dict())
            _dict['tools'] = _items
        # set to None if model_client (nullable) is None
        # and model_fields_set contains the field
        if self.model_client is None and "model_client" in self.model_fields_set:
            _dict['model_client'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of MultimodalWebSurferAgentConfig from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "component_type": obj.get("component_type"),
            "version": obj.get("version"),
            "description": obj.get("description"),
            "name": obj.get("name"),
            "agent_type": obj.get("agent_type"),
            "system_message": obj.get("system_message"),
            "model_client": obj.get("model_client"),
            "tools": [ToolConfig.from_dict(_item) for _item in obj["tools"]] if obj.get("tools") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Mtmai API

    The Mtmai API

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool
from typing import Any, ClassVar, Dict, List, Optional
from mtmai.gomtmclients.rest.models.crew_ai_agent import CrewAiAgent
from mtmai.gomtmclients.rest.models.crew_ai_task import CrewAiTask
from mtmai.gomtmclients.rest.models.llm_config import LlmConfig
from typing import Optional, Set
from typing_extensions import Self

class CallAgent(BaseModel):
    """
    调用 Agent 参数
    """ # noqa: E501
    llm: Optional[LlmConfig] = Field(default=None, description="大语言模型 api 配置")
    input: Dict[str, Any] = Field(description="输入参数")
    agents: List[CrewAiAgent] = Field(description="agents 列表")
    tasks: List[CrewAiTask] = Field(description="任务列表")
    debug: Optional[StrictBool] = Field(default=False, description="是否调试模式")
    __properties: ClassVar[List[str]] = ["llm", "input", "agents", "tasks", "debug"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CallAgent from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of llm
        if self.llm:
            _dict['llm'] = self.llm.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in agents (list)
        _items = []
        if self.agents:
            for _item_agents in self.agents:
                if _item_agents:
                    _items.append(_item_agents.to_dict())
            _dict['agents'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in tasks (list)
        _items = []
        if self.tasks:
            for _item_tasks in self.tasks:
                if _item_tasks:
                    _items.append(_item_tasks.to_dict())
            _dict['tasks'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CallAgent from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "llm": LlmConfig.from_dict(obj["llm"]) if obj.get("llm") is not None else None,
            "input": obj.get("input"),
            "agents": [CrewAiAgent.from_dict(_item) for _item in obj["agents"]] if obj.get("agents") is not None else None,
            "tasks": [CrewAiTask.from_dict(_item) for _item in obj["tasks"]] if obj.get("tasks") is not None else None,
            "debug": obj.get("debug") if obj.get("debug") is not None else False
        })
        return _obj





import asyncio
import logging

from mtmlib import env
from mtmlib.queue.mtqueue_client import MtQueueClient
from mtmlib.queue.queue import MessagePublic

from mtmtrain.core.config import settings

logger = logging.getLogger("mttrain_worker")


def worker_start():
    logger.info("worker start")
    if env.in_jupyter_notebook():
        print(" run @ ipython")
    queue_client = MtQueueClient(backend=settings.MTMAI_API_BASE)

    queue_client.register_consumer(queue_name="test1", consumer_fn=msg_handler1)
    asyncio.run(queue_client.run())

    logger.info("worker end!")


def msg_handler1(msg: MessagePublic):
    logger.info("msg_handler1 处理消息 %s", msg)



from fastapi import APIRouter

# from mtmaisdk.clients.rest import AgentNodeRunRequest

# from pydantic import BaseModel

router = APIRouter()

# @router.post("/message")
# async def message(data: MessageAckRequest):
#     """This endpoint is called by the client to start a message generation workflow."""
#     workflowRun = await hatchet.client.admin.aio.run_workflow(
#         "BasicRagWorkflow", {"request": data.model_dump()}
#     )

#     # normally, we'd save the workflow_run_id to a database and return a reference to the client
#     # for this simple example, we just return the workflow_run_id

#     return {"messageId": workflowRun.workflow_run_id}


# async def event_stream_generator(workflowRunId):
#     """This helper function is a generator that yields events from the Hatchet event stream."""
#     workflowRun = hatchet.client.admin.get_workflow_run(workflowRunId)

#     async for event in workflowRun.stream():
#         """ you can filter and transform event data here that will be sent to the client"""
#         data = json.dumps(
#             {"type": event.type, "payload": event.payload, "messageId": workflowRunId}
#         )
#         yield "data: " + data + "\n\n"

#     result = await workflowRun.result()

#     data = json.dumps({"type": "result", "payload": result, "messageId": workflowRunId})

#     yield "data: " + data + "\n\n"


@router.get("/message/{messageId}")
async def stream(messageId: str):
    """
    in a normal application you might use the message id to look up a workflowRunId
    for this simple case, we have no persistence and just use the message id as the workflowRunId

    you might also consider looking up the workflowRunId in a database and returning the results if the message has already been processed
    """
    # workflowRunId = messageId
    # return StreamingResponse(
    #     event_stream_generator(workflowRunId), media_type="text/event-stream"
    # )
    pass


# @router.get("/hello/ag")
# async def hello_ag():
#     from mtmai.gomtmclients.rest import ApiClient

#     print("ApiClient")
#     print(ApiClient)
#     return {
#         "hello": "ag",
#     }



from pydantic import BaseModel


async def emit_flow_event(event: str, resource_id: str, data: dict | BaseModel = {}):
    from prefect.events import emit_event

    _data = data.model_dump() if isinstance(data, BaseModel) else data
    event = emit_event(
        event=event,
        resource={
            **_data,
            "prefect.resource.id": str(resource_id),
        },
    )
    return event



"""
MergeAnswersNode Module
"""

from typing import List, Optional

from langchain_core.runnables import RunnableConfig

from .base_node import BaseNode


class MergeAnswersNode(BaseNode):
    """
    一个负责将多个图实例的答案合并成单一答案的节点。
    A node responsible for merging the answers from multiple graph instances into a single answer.

    Attributes:
        llm_model: An instance of a language model client, configured for generating answers.
        verbose (bool): A flag indicating whether to show print statements during execution.

    Args:
        input (str): Boolean expression defining the input keys needed from the state.
        output (List[str]): List of output keys to be updated in the state.
        node_config (dict): Additional configuration for the node.
        node_name (str): The unique identifier name for the node, defaulting to "GenerateAnswer".
    """

    def __init__(
        self,
        input: str,
        output: List[str],
        node_config: Optional[dict] = None,
        node_name: str = "MergeAnswers",
    ):
        super().__init__(node_name, "node", input, output, 2, node_config)

        self.llm_model = node_config["llm_model"]
        self.verbose = (
            False if node_config is None else node_config.get("verbose", False)
        )

    async def __call__(self, state: dict, config: RunnableConfig):
        pass



import uuid
from datetime import datetime

from sqlmodel import Field, SQLModel


class TaggedItem(SQLModel, table=True):
    id: uuid.UUID = Field(default_factory=uuid.uuid4, primary_key=True)
    tag_id: uuid.UUID = Field(foreign_key="tag.id")
    item_id: uuid.UUID
    item_type: str


class MtkvBase(SQLModel):
    key: str = Field(primary_key=True, index=True)
    value: str
    created_at: datetime = Field(default_factory=datetime.now, nullable=False)
    updated_at: datetime = Field(default_factory=datetime.now, nullable=False)
    expired_at: datetime | None = Field(default=None)


class Mtkv(MtkvBase, table=True):
    pass



import asyncio
import os
import sys
from time import sleep

import structlog
from mtmaisdk import ClientConfig, Hatchet, loader
from mtmaisdk.clients.rest import ApiClient
from mtmaisdk.clients.rest.api.mtmai_api import MtmaiApi
from mtmaisdk.clients.rest.configuration import Configuration

from mtmai.core.config import settings

wfapp: Hatchet = None


class WorkerApp:
    def __init__(self, backend_url: str | None):
        self.backend_url = backend_url
        if not self.backend_url:
            raise ValueError("backend_url is not set")
        self.log = structlog.get_logger()

    async def setup(self):
        global wfapp

        self.api_client = ApiClient(
            configuration=Configuration(
                host=self.backend_url,
            )
        )

        maxRetry = 50
        interval = 5
        for i in range(maxRetry):
            try:
                self.log.info("connectting...")
                mtmaiapi = MtmaiApi(self.api_client)
                workerConfig = await mtmaiapi.mtmai_worker_config()
                os.environ["HATCHET_CLIENT_TLS_STRATEGY"] = "none"
                os.environ["HATCHET_CLIENT_TOKEN"] = workerConfig.token
                os.environ["DISPLAY"] = ":1"
                config_loader = loader.ConfigLoader(".")
                cc = config_loader.load_client_config(
                    ClientConfig(
                        server_url=settings.GOMTM_URL,
                        host_port=workerConfig.grpc_host_port,
                        tls_config=loader.ClientTLSConfig(
                            tls_strategy="none",
                            cert_file="None",
                            key_file="None",
                            ca_file="None",
                            server_name="localhost",
                        ),
                    )
                )
                wfapp = Hatchet.from_config(cc, debug=True)
                return wfapp
            except Exception as e:
                self.log.error(f"failed to create hatchet: {e}")
                if i == maxRetry - 1:
                    sys.exit(1)
                sleep(interval)
        raise ValueError("failed to connect gomtm server")

    async def deploy_mtmai_workers(self):
        await self.setup()
        self.log.info("start worker")
        worker = wfapp.worker("pyworker")
        if not worker:
            raise ValueError("worker not found")

        from mtmai.workflows.flow_assistant import FlowAssistant

        worker.register_workflow(FlowAssistant())
        from mtmai.workflows.flow_crewai import FlowCrewAIAgent

        worker.register_workflow(FlowCrewAIAgent())

        from mtmai.workflows.flow_browser import FlowBrowser

        worker.register_workflow(FlowBrowser())
        await worker.async_start()

        while True:
            await asyncio.sleep(1)



from typing import Any

import httpx
import structlog
from fastapi import APIRouter, Depends, HTTPException, Response
from sqlmodel import col, delete, func, select

from mtmai.core.config import settings
from mtmai.core.security import get_password_hash, verify_password
from mtmai.crud import curd
from mtmai.crud.curd_account import get_account_by_user_id
from mtmai.deps import (
    AsyncSessionDep,
    CurrentUser,
    OptionalUserDep,
    SessionDep,
    get_current_active_superuser,
)
from mtmai.models.models import (
    Item,
    Message,
    UpdatePassword,
    User,
    UserCreate,
    UserPublic,
    UserRegister,
    UsersPublic,
    UserUpdate,
    UserUpdateMe,
)

# from mtmai.utils import generate_new_account_email, send_email
# from mtmlib.github import get_github_user_data

router = APIRouter()

# logger = logging.getLogger()
LOG = structlog.get_logger()


@router.get(
    "/",
    dependencies=[Depends(get_current_active_superuser)],
    response_model=UsersPublic,
)
def read_users(session: SessionDep, skip: int = 0, limit: int = 100) -> Any:
    """
    Retrieve users.
    """
    count_statement = select(func.count()).select_from(User)
    count = session.exec(count_statement).one()

    statement = select(User).offset(skip).limit(limit)
    users = session.exec(statement).all()

    return UsersPublic(data=users, count=count)


@router.post(
    "/", dependencies=[Depends(get_current_active_superuser)], response_model=UserPublic
)
async def create_user(*, session: AsyncSessionDep, user_in: UserCreate) -> Any:
    """
    Create new user.
    """
    user = await curd.get_user_by_email(session=session, email=user_in.email)
    if user:
        raise HTTPException(
            status_code=400,
            detail="The user with this email already exists in the system.",
        )

    user = curd.create_user(session=session, user_create=user_in)
    if settings.emails_enabled and user_in.email:
        email_data = generate_new_account_email(
            email_to=user_in.email, username=user_in.email, password=user_in.password
        )
        send_email(
            email_to=user_in.email,
            subject=email_data.subject,
            html_content=email_data.html_content,
        )
    return user


@router.patch("/me", response_model=UserPublic)
async def update_user_me(
    *, session: SessionDep, user_in: UserUpdateMe, current_user: CurrentUser
) -> Any:
    """Update own user."""
    if user_in.email:
        existing_user = await curd.get_user_by_email(
            session=session, email=user_in.email
        )
        if existing_user and existing_user.id != current_user.id:
            raise HTTPException(
                status_code=409, detail="User with this email already exists"
            )
    user_data = user_in.model_dump(exclude_unset=True)
    current_user.sqlmodel_update(user_data)
    session.add(current_user)
    session.commit()
    session.refresh(current_user)
    return current_user


@router.patch("/me/password", response_model=Message)
def update_password_me(
    *, session: SessionDep, body: UpdatePassword, current_user: CurrentUser
) -> Any:
    """
    Update own password.
    """
    if not verify_password(body.current_password, current_user.hashed_password):
        raise HTTPException(status_code=400, detail="Incorrect password")
    if body.current_password == body.new_password:
        raise HTTPException(
            status_code=400, detail="New password cannot be the same as the current one"
        )
    hashed_password = get_password_hash(body.new_password)
    current_user.hashed_password = hashed_password
    session.add(current_user)
    session.commit()
    return Message(message="Password updated successfully")


@router.get("/me", response_model=UserPublic | None)
async def read_user_me(db: SessionDep, user: OptionalUserDep):
    """
    Get current user.
    """
    # response = UserPublic(**user.model_dump())
    # return response
    return user


@router.delete("/me", response_model=Message)
async def delete_user_me(db: SessionDep, current_user: CurrentUser) -> Any:
    """
    Delete own user.
    """
    if current_user.is_superuser:
        raise HTTPException(
            status_code=403, detail="Super users are not allowed to delete themselves"
        )
    statement = delete(Item).where(col(Item.owner_id) == current_user.id)
    db.exec(statement)  # type: ignore
    db.delete(current_user)
    db.commit()
    return Message(message="User deleted successfully")


@router.post("/signup", response_model=UserPublic)
async def register_user(session: SessionDep, user_in: UserRegister) -> Any:
    """
    Create new user without the need to be logged in.
    """
    return await curd.register_user(session=session, user_in=user_in)


@router.get("/avatar", include_in_schema=False)
async def avatar(db: SessionDep, user: OptionalUserDep):
    """
    获取用户头像
    """
    if not user:
        raise HTTPException(status_code=404, detail="User not found")

    account = get_account_by_user_id(session=db, owner_id=user.id)
    if not account:
        raise HTTPException(status_code=404, detail="Account not found")

    github_user_data = await get_github_user_data(account.token)
    if not github_user_data:
        raise HTTPException(status_code=404, detail="GitHub user data not found")

    avatar_url = github_user_data.get("avatar_url")
    if not avatar_url:
        LOG.info(f"找不到 用户头像 {github_user_data}")
        LOG.info(f"account {account}")
        raise HTTPException(status_code=404, detail="Avatar URL not found")

    async with httpx.AsyncClient() as client:
        avatar_response = await client.get(avatar_url)
        if avatar_response.status_code != 200:
            raise HTTPException(status_code=404, detail="Failed to fetch avatar")
    return Response(
        content=avatar_response.content,
        media_type=avatar_response.headers.get("content-type"),
        headers={"Cache-Control": "public, max-age=3600"},
    )


@router.get("/{user_id}", response_model=UserPublic)
def read_user_by_id(
    user_id: str,
    session: SessionDep,
    current_user: CurrentUser,
) -> Any:
    """
    Get a specific user by id.
    """
    user = session.get(User, user_id)
    if user == current_user:
        return user
    if not current_user.is_superuser:
        raise HTTPException(
            status_code=403,
            detail="The user doesn't have enough privileges",
        )
    return user


@router.patch(
    "/{user_id}",
    dependencies=[Depends(get_current_active_superuser)],
    response_model=UserPublic,
)
async def update_user(
    *,
    session: SessionDep,
    user_id: str,
    user_in: UserUpdate,
) -> Any:
    """
    Update a user.
    """
    db_user = session.get(User, user_id)
    if not db_user:
        raise HTTPException(
            status_code=404,
            detail="The user with this id does not exist in the system",
        )
    if user_in.email:
        existing_user = await curd.get_user_by_email(
            session=session, email=user_in.email
        )
        if existing_user and existing_user.id != user_id:
            raise HTTPException(
                status_code=409, detail="User with this email already exists"
            )

    db_user = curd.update_user(session=session, db_user=db_user, user_in=user_in)
    return db_user


@router.delete("/{user_id}", dependencies=[Depends(get_current_active_superuser)])
def delete_user(
    session: SessionDep, current_user: CurrentUser, user_id: str
) -> Message:
    """
    Delete a user.
    """
    user = session.get(User, user_id)
    if not user:
        raise HTTPException(status_code=404, detail="User not found")
    if user == current_user:
        raise HTTPException(
            status_code=403, detail="Super users are not allowed to delete themselves"
        )
    statement = delete(Item).where(col(Item.owner_id) == user_id)
    session.exec(statement)  # type: ignore
    session.delete(user)
    session.commit()
    return Message(message="User deleted successfully")



import logging

from langchain_core.messages import ChatMessage
from langchain_core.runnables import Runnable, RunnableConfig

from mtmai.agents.states.state import MainState

logger = logging.getLogger()


def edge_mtmeditor(state: MainState):
    return "uidelta"


class MtmEditorNode:
    def __init__(self, runnable: Runnable):
        self.runnable = runnable

    async def __call__(self, state: MainState, config: RunnableConfig):
        thread_id = config.get("configurable").get("thread_id")

        user_input = state.get("user_input")
        user_option = state.get("user_option")
        if user_option:
            # 是可视化编辑器的指令
            return {"go_mtmeditor": True}
        if user_input == "":
            logger.info("前端页面刷新后重新获取状态")
        if user_input.startswith("/"):
            return await self.uicommand(state, config)

        messages = state.get("messages")
        if len(messages) < 1:
            raise Exception("消息长度不正确")  # noqa: EM101, TRY002
        llm = self.runnable
        if messages[-1].type == "tool":
            ai_message = await llm.ainvoke(messages, config)
            return {
                "messages": [
                    ai_message,
                ],
            }
        new_user_message = ChatMessage(role="user", content=state.get("user_input"))
        messages.append(new_user_message)
        ai_message = await llm.ainvoke(messages, config)
        finnal_state = {
            "messages": [
                new_user_message,
                ai_message,
            ],
            "uidelta": state.get("uidelta"),
        }
        return finnal_state

    async def uicommand(self, state: MainState, config: RunnableConfig):
        user_input = state.get("user_input")
        return {}



# coding: utf-8

"""
    Mtmai API

    The Mtmai API

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class ProgrammingLanguageOptions(str, Enum):
    """
    ProgrammingLanguageOptions
    """

    """
    allowed enum values
    """
    TYPESCRIPT = 'typescript'
    JAVASCRIPT = 'javascript'
    CPP = 'cpp'
    JAVA = 'java'
    PHP = 'php'
    PYTHON = 'python'
    HTML = 'html'
    SQL = 'sql'
    JSON = 'json'
    RUST = 'rust'
    XML = 'xml'
    CLOJURE = 'clojure'
    CSHARP = 'csharp'
    OTHER = 'other'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ProgrammingLanguageOptions from a JSON string"""
        return cls(json.loads(json_str))





from datetime import datetime
from typing import Any, Dict, List, Optional

from autogen_agentchat.base import TaskResult
from autogen_core import ComponentModel
from pydantic import BaseModel


class MessageConfig(BaseModel):
    source: str
    content: str
    message_type: Optional[str] = "text"


class TeamResult(BaseModel):
    task_result: TaskResult
    usage: str
    duration: float


class MessageMeta(BaseModel):
    task: Optional[str] = None
    task_result: Optional[TaskResult] = None
    summary_method: Optional[str] = "last"
    files: Optional[List[dict]] = None
    time: Optional[datetime] = None
    log: Optional[List[dict]] = None
    usage: Optional[List[dict]] = None


class GalleryMetadata(BaseModel):
    author: str
    created_at: datetime
    updated_at: datetime
    version: str
    description: Optional[str] = None
    tags: Optional[List[str]] = None
    license: Optional[str] = None
    homepage: Optional[str] = None
    category: Optional[str] = None
    last_synced: Optional[datetime] = None


class GalleryComponents(BaseModel):
    agents: List[ComponentModel]
    models: List[ComponentModel]
    tools: List[ComponentModel]
    terminations: List[ComponentModel]


class GalleryItems(BaseModel):
    teams: List[ComponentModel]
    components: GalleryComponents


class Gallery(BaseModel):
    id: str
    name: str
    url: Optional[str] = None
    metadata: GalleryMetadata
    items: GalleryItems


# web request/response data models


class Response(BaseModel):
    message: str
    status: bool
    data: Optional[Any] = None


class SocketMessage(BaseModel):
    connection_id: str
    data: Dict[str, Any]
    type: str



import os
import warnings
from typing import Annotated, Any, Literal

from pydantic import (AnyUrl, BeforeValidator, HttpUrl, computed_field,
                      model_validator)
from pydantic_settings import BaseSettings, SettingsConfigDict
from typing_extensions import Self

from mtmlib import mtutils

from .__version__ import version


def parse_cors(v: Any) -> list[str] | str:
    if isinstance(v, str) and not v.startswith("["):
        return [i.strip() for i in v.split(",")]
    if isinstance(v, list | str):
        return v
    raise ValueError(v)


class Settings(BaseSettings):
    model_config = SettingsConfigDict(
        env_file=".env", env_ignore_empty=True, extra="ignore"
    )
    app_name: str = "Mtmai API"
    PORT: int | None = 8000
    # admin_email: str | None = None
    items_per_user: int = 50
    # SECRET_KEY: str = secrets.token_urlsafe(32)
    SECRET_KEY: str = "09d25e094faa6ca2556c818166b7a9563b93f7099f6f0f4caa6cf63b88e8d3e7"

    # oauth
    algorithm: str = "HS256"
    access_token_expire_minutes: int = 30
    ACCESS_TOKEN_EXPIRE_MINUTES: int = 60 * 24 * 8

    DOMAIN: str = "localhost"
    ENVIRONMENT: Literal["local", "staging", "production"] = "local"

    @computed_field  # type: ignore[prop-decorator]
    @property
    def server_host(self) -> str:
        # Use HTTPS for anything other than local development
        if self.ENVIRONMENT == "local":
            return f"http://{self.DOMAIN}"
        return f"https://{self.DOMAIN}"

    BACKEND_CORS_ORIGINS: Annotated[
        list[AnyUrl] | str, BeforeValidator(parse_cors)
    ] = []

    # db
    DATABASE_URL: str | None = None  # config("DATABASE_URL", cast=Secret)
    DATABASE_URL_2: str | None = None

    API_V1_STR: str = "/api/v1"
    VERSION: str | None = version

    vercel_token: str | None = None

    PROJECT_NAME: str = "mtmai"
    SENTRY_DSN: HttpUrl | None = None
    POSTGRES_SERVER: str = "POSTGRES_SERVER"
    POSTGRES_PORT: int = 5432
    POSTGRES_USER: str = "POSTGRES_USER"
    POSTGRES_PASSWORD: str = ""
    POSTGRES_DB: str = ""

    SMTP_TLS: bool = True
    SMTP_SSL: bool = False
    SMTP_PORT: int = 587
    SMTP_HOST: str | None = None
    SMTP_USER: str | None = None
    SMTP_PASSWORD: str | None = None
    # TODO: update type to EmailStr when sqlmodel supports it
    EMAILS_FROM_EMAIL: str | None = None
    EMAILS_FROM_NAME: str | None = None

    # cloudflare
    CLOUDFLARE_ACCOUNT_ID: str | None = None
    CLOUDFLARE_API_EMAIL: str | None = None
    CLOUDFLARE_API_TOKEN: str | None = None
    CLOUDFLARE_AI_TOKEN: str | None = None

    # tembo
    TEMBO_TOKEN: str | None = None
    TEMBO_ORG: str | None = None
    TEMBO_INST: str | None = None
    TEMBO_DATA_DOMAIN: str | None = None

    # logging
    LOGGING_LEVEL: str | None = "info"
    LOGGING_PATH: str | None = ""
    LOGGING_FORMAT: str | None = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"

    # cloudflared tunnel
    CF_TUNNEL_TOKEN: str | None = None
    CF_TUNNEL_TOKEN_TEMBO: str | None = None
    CF_TUNNEL_TOKEN_HF: str | None = None

    @computed_field  # type: ignore[prop-decorator]
    @property
    def emails_enabled(self) -> bool:
        return bool(self.SMTP_HOST and self.EMAILS_FROM_EMAIL)

    EMAIL_TEST_USER: str = "test@example.com"
    FIRST_SUPERUSER: str = "mt@mt.com"
    FIRST_SUPERUSER_PASSWORD: str = "feihuo321"
    FIRST_SUPERUSER_EMAIL: str = "mt@mt.com"

    def _check_default_secret(self, var_name: str, value: str | None) -> None:
        if value == "changethis":
            message = (
                f'The value of {var_name} is "changethis", '
                "for security, please change it, at least for deployments."
            )
            if self.ENVIRONMENT == "local":
                warnings.warn(message, stacklevel=1)
            else:
                raise ValueError(message)

    @model_validator(mode="after")
    def _enforce_non_default_secrets(self) -> Self:
        self._check_default_secret("SECRET_KEY", self.SECRET_KEY)
        self._check_default_secret("POSTGRES_PASSWORD", self.POSTGRES_PASSWORD)
        self._check_default_secret(
            "FIRST_SUPERUSER_PASSWORD", self.FIRST_SUPERUSER_PASSWORD
        )

        return self

    @computed_field  # type: ignore[prop-decorator]
    @property
    def is_in_gitpod(self) -> bool:
        return os.getenv("GITPOD_WORKSPACE_URL")

    @computed_field  # type: ignore[prop-decorator]
    @property
    def is_in_vercel(self) -> bool:
        return os.getenv("VERCEL")

    SEARXNG_URL_BASE: str | None = "http://127.0.0.1:18777"

    MAIN_GH_TOKEN: str | None = None
    MAIN_GH_USER: str | None = None

    DEFAULT_PASSWORD: str | None = "feihuo321"

    # huggingface
    HUGGINGFACEHUB_API_TOKEN: str | None = None
    HUGGINGFACEHUB_USER: str | None = None
    HUGGINGFACEHUB_DEFAULT_WORKSPACE: str | None = None

    gitsrc_dir: str | None = "gitsrc"

    IS_TRACE_HTTPX: bool = True
    OTEL_ENABLED: bool | None = False

    LOKI_ENDPOINT: str | None = "https://logs-prod-017.grafana.net/loki/api/v1/push"
    LOKI_USER: str | None = None
    GRAFANA_TOKEN: str | None = None

    # front
    # FRONT_PORT: int = 3800

    # POETRY_PYPI_TOKEN_PYPI: str | None = None

    MTMAI_API_BASE: str | None = None

    # storage
    @computed_field  # type: ignore[prop-decorator]
    @property
    def storage_dir(self) -> str:  # str | None = "/app/storage"
        if mtutils.is_in_gitpod() or mtutils.in_colab:
            return ".vol"

        return "/app/storage"


settings = Settings()  # type: ignore



from typing import Annotated

from langchain_core.messages import HumanMessage
from langchain_core.prompts import ChatPromptTemplate, MessagesPlaceholder
from langchain_core.runnables import Runnable, RunnableConfig
from langchain_core.tools import tool
from langgraph.prebuilt import InjectedState, tools_condition

from mtmai.agents.graphchatdemo.tools.search_tools import search_tool
from mtmai.core.config import settings
from mtmai.core.logging import get_logger
from mtmai.models.graph_config import HomeChatState

logger = get_logger()



class ArticleEditorNode:
    def __init__(self, runnable: Runnable):
        self.runnable = runnable

    async def __call__(self, state: HomeChatState, config: RunnableConfig):
        thread_ui_state = state.thread_ui_state
        return {
            "thread_ui_state": {
                "playData": {
                        "title": "Hello World2222",
                        "content": "This is a test post",
                },
            },
        }



from typing import Any

import httpx
from bs4 import BeautifulSoup
from prefect import flow, get_run_logger, task
from pydantic import BaseModel

# from mtmai.biz import biz_scheule
from mtmai.crud import crud_task
from mtmai.crud.curd_search import create_site_search_index
from mtmai.crud.curd_site import get_site_by_id, get_sites_enabled_automation
from mtmai.db.db import get_async_session
from mtmai.deps import AsyncSessionDep
from mtmai.flows import FlowBase, mtflow
from mtmai.flows.article_gen import (
    WriteSingleChapterRequest,
    article_gen_outline,
    write_book_chapter_crew,
)
from mtmai.flows.flow_publish import publish_article
from mtmai.flows.flows_util import mtflow_util
from mtmai.models.book_gen import (
    Chapter,
    GenBookState,
    WriteOutlineRequest,
)
from mtmai.models.site import (
    SiteCreateRequest,
)
from mtmai.models.task import MtTaskType


class SiteDetectInfo(BaseModel):
    title: str | None = None
    description: str | None = None


@task()
async def site_info_detect(session: AsyncSessionDep, url: str):
    """获取远程站点基本信息"""
    async with httpx.AsyncClient() as client:
        resp = await client.get(url)

    soup = BeautifulSoup(resp.text, "html.parser")
    title = soup.title.string if soup.title else None
    meta_description = soup.find("meta", attrs={"name": "description"})
    description = meta_description["content"] if meta_description else None
    if description:
        description = description[:100]
    return SiteDetectInfo(title=title, description=description)


@task()
async def create_site_task(
    site_id: str,
    user_id: str,
):
    logger = get_run_logger()
    try:
        logger.info(f"create_site_task 开始，site_id: {site_id}, user_id: {user_id}")
        async with get_async_session() as session:
            site = await get_site_by_id(session, site_id, user_id)
        if not site:
            logger.info(f"site_id: {site_id}, user_id: {user_id} 不存在")

        logger.info(f"工作流开始更新 site 信息 {site_id}")
        target_url = str(site.url)
        site_info = await site_info_detect(session, target_url)

        # Convert item_in to dict, convert url to string, and add owner_id
        site_data = site.model_dump()
        site_data["url"] = str(site_data["url"])  # Convert Url to string
        site_data["owner_id"] = user_id
        site.title = site_info.title
        site.description = site_info.description
        session.add(site)
        await session.commit()
        await session.refresh(site)
        await create_site_search_index(session, site, user_id)
        await session.refresh(site)
        ret = site.model_dump()
        logger.info(f"site_id: {site_id}, user_id: {user_id} 更新完成")
    except Exception as e:
        logger.error(
            f"site_id: {site_id}, user_id: {user_id} create_site_task失败: {e}"
        )
    return ret


@mtflow(SiteCreateRequest)
class CreateSiteFlow(FlowBase):
    @classmethod
    @flow(name="CreateSiteFlow")
    async def execute(cls, site_id: str, user_id: str) -> Any:
        # yield aisdk.AiTextChunck("<mtmai_response>\n")
        logger = get_run_logger()
        logger.info(f"site_id: {site_id}, user_id: {user_id}")
        async with get_async_session() as session:
            site = await get_site_by_id(session, site_id, user_id)
        if not site:
            logger.info(f"site_id: {site_id}, user_id: {user_id} 不存在")

        logger.info("工作流开始更新 site 信息")
        # try:
        #     yield aisdk.AiTextChunck("<mtmai_msg>开始处理</mtmai_msg>\n")
        #     req_model: SiteCreateRequest = cls.form_model
        #     yield aisdk.AiTextChunck("<mtmai_msg>验证数据</mtmai_msg>\n")
        #     item_in = req_model.model_validate(data)
        #     yield aisdk.AiTextChunck("<mtmai_msg>调用工作流</mtmai_msg>\n")
        #     task1_result = await create_site_task(item_in, user_id)
        #     yield aisdk.AiTextChunck("<mtmai_msg>完成</mtmai_msg>\n")
        #     yield aisdk.AiTextChunck(
        #         '<mtmai_action url="https://www.baidu.com">自动跳转</mtmai_action>\n'
        #     )
        # except ValidationError as e:
        #     yield aisdk.AiTextChunck(f"输入不正确: {e}")
        #     # pass
        # yield aisdk.AiTextChunck("</mtmai_response>\n")


@flow
async def create_site_flow(user_id: str, site_id: str):
    logger = get_run_logger()
    logger.info(f"create_site_flow user_id: {user_id}, site_id: {site_id}")

    await create_site_task(site_id, user_id)


@flow
async def flow_site_automation():
    """后台检测 site 状态，根据状态自动触发子工作流的运行"""
    async with mtflow_util() as flow_util:
        flow_util.info("开始检测 site 状态")
        async with get_async_session() as session:
            sites = await get_sites_enabled_automation(session)

        if not sites or len(sites) == 0:
            flow_util.info("没有站点启用自动化")
            return

        for site in sites:
            flow_util.info(f"开始调度 site {site.id} 的任务")
            async with get_async_session() as session:
                tasks_to_run = await crud_task.get_tasks_to_run(
                    session=session, site_id=site.id, limit=1
                )

            if not tasks_to_run or len(tasks_to_run) == 0:
                flow_util.info("没有任务需要运行, 现在创建一个文章生成任务")
                async with get_async_session() as session:
                    # TODO: 这里的参数应该 AI 生成

                    # init_state = {
                    #     "topic": "AI 在 2024 年 9 月的现状: 各行业的趋势和未来展望",
                    #     "goal": "生成一本书，介绍 AI 在 2024 年 9 月的现状，包括各行业的趋势和未来展望。",
                    # }
                    # new_mttask = await mttask_create(
                    #     session=session,
                    #     site_id=site.id,
                    #     name="gen_article",
                    #     init_state={},
                    # )
                    # await flow_run_task(str(new_mttask.id))
                    pass

            else:
                flow_util.info(f"有任务需要运行, len={len(tasks_to_run)}")
                for ta in tasks_to_run:
                    flow_util.info(f"开始执行任务: {ta}")
                    await flow_run_task(str(ta.id))

        # 新方式：根据任务计划表进行调度
        async with get_async_session() as session:
            schedules = await crud_task.list_schedult_to_run(session=session)
            flow_util.info(f"调度任务计划数量: {len(schedules)}")
            for schedule in schedules:
                await flow_run_schedule(str(schedule.id))


@flow
async def flow_run_schedule(schedule_id: str):
    """根据数据库表 schedule 的值启动对应的工作流"""
    async with mtflow_util() as flow_util:
        flow_util.info(f"开始运行 schedule {schedule_id}")
        # async with get_async_session() as session:
        #     sched = await crud_task.get_schedule(session=session, id=schedule_id)
        #     if not sched:
        #         raise ValueError(f"schedule {schedule_id} 不存在")

        #     task_to_run = None
        #     if sched.task_type == MtTaskType.ARTICLE_GEN:
        #         task_to_run = await mttask_create(
        #             session=session,
        #             schedule_id=schedule_id,
        #             name=MtTaskType.ARTICLE_GEN,
        #             init_state={
        #                 **sched.params,
        #             },
        #         )
        #     else:
        #         raise ValueError(f"未知的任务类型 {sched.task_type}")

        new_task = await biz_scheule.make_new_task_by_schedule(schedule_id)

        await flow_run_task(str(new_task.id))


@flow
async def flow_run_task(mttask_id: str):
    """根据数据库表 mttask 的值启动对应的工作流"""
    async with mtflow_util() as flow_util:
        async with get_async_session() as session:
            mttask = await crud_task.mttask_get_by_id(
                session=session, mttask_id=mttask_id
            )
        flow_util.info(f"开始运行mttask {mttask.id}")

        if not mttask:
            raise ValueError(f"mttask {mttask_id} 不存在")
        if not mttask.name:
            raise ValueError(f"mttask {mttask.id} 没有 name 值")

        # 状态判断
        if mttask.status == "new":
            flow_util.info(f"mttask {mttask.id} 状态为 new, 跳过")
            return

        task_name = mttask.name
        if mttask.status == "pending":
            match task_name:
                case MtTaskType.ARTICLE_GEN:
                    await flow_run_gen_article(mttask_id=str(mttask.id))
                case _:
                    raise ValueError(f"未知的任务类型 {mttask.name}")

            flow_util.info(f"完成 mttask {mttask.id}")


@flow
async def flow_run_gen_article(mttask_id: str):
    """
    单个站点的自动化工作流
    """
    async with mtflow_util() as flow_util:
        await flow_util.info(f"flow_site_gen mttask_id: {mttask_id}")

        async with get_async_session() as session:
            mttask = await crud_task.mttask_get_by_id(
                session=session, mttask_id=mttask_id
            )
            if not mttask:
                raise ValueError(f"mttask {mttask_id} 不存在")
            sched = await crud_task.schedule_get_by_id(
                session=session, schedule_id=mttask.schedule_id
            )
            site_id = sched.inputs.get("site_id")
            if not site_id:
                raise ValueError(f"schedule {sched.id} 没有 site_id 值")
            site = await get_site_by_id(session=session, site_id=site_id)

        flow_util.info("生成大纲 ...")
        async with get_async_session() as session:
            mttask = await crud_task.mttask_get_by_id(
                session=session, mttask_id=mttask_id
            )

        _state = mttask.state or {}
        state = GenBookState.model_validate(_state)

        if not state.title and state.topic:
            # state 为全新态
            # TODO: 这里的初始参数应该 由 AI 生成
            state.title = "AI 在 2024 年 9 月的现状: 各行业的趋势和未来展望"
            state.goal = "生成一本书，介绍 AI 在 2024 年 9 月的现状，包括各行业的趋势和未来展望。"

        outlines = await article_gen_outline(
            req=WriteOutlineRequest(
                topic=state.topic,
                goal=state.goal,
            )
        )
        await flow_util.info(f"flow_article_gen end, outlines: {outlines}")

        if not outlines:
            flow_util.info("大纲编写失败")
            return
        await flow_util.info("大纲编写完成，开始写内容")

        async with get_async_session() as session:
            await crud_task.mttask_update_state(
                session=session, mttask_id=mttask_id, state=state
            )

        chapters = outlines.chapters
        state.book_outline = chapters

        chapters = []
        for index, chapter_outline in enumerate(state.book_outline, start=1):
            flow_util.info(
                f"开始写章节正文 {index}/{len(state.book_outline)}: {chapter_outline.title}"
            )
            output = await write_book_chapter_crew(
                req=WriteSingleChapterRequest(
                    goal=state.goal,
                    topic=state.topic,
                    chapter_title=chapter_outline.title,
                    chapter_description=chapter_outline.description,
                    book_outlines=state.book_outline,
                )
            )
            if not output:
                await flow_util.info(f"写章节正文失败: {output}")
                continue
            chapters.append(Chapter(title=output.title, content=output.content))

        state.book.extend(chapters)
        # sections = [await write_section(topic) for topic in outline]
        await flow_util.info(f"章节内容写入完成, 章节数量:{len(chapters)}")
        await publish_article(site_id=site.id, state=state)
    # except litellm.RateLimitError as e:
    #     flow_util.error(f"调用大模型到达限制，TODO 切换大模型: {e}")
    #     raise e
    # except Exception as e:
    #     flow_util.error(f"flow_site_gen 失败: {e}")
    #     raise e



"""站点 curd 操作"""

import uuid

from sqlmodel import select
from sqlmodel.ext.asyncio.session import AsyncSession

from mtmai.crud.curd_search import create_site_search_index
from mtmai.models.site import (
    Site,
    SiteCreateRequest,
    SiteHost,
)
# from mtmai.models.task import TaskSchedule


async def get_site_by_id(session: AsyncSession, site_id: uuid.UUID | str):
    if site_id == "testing":
        testing_site = Site(
            id=uuid.uuid4(),
            name="测试站点",
            description="这是一个测试站点",
            # gen_config=TaskSchedule(
            #     site_url="https://www.example.com",
            #     site_topic="AI",
            #     description="AI 相关站点",
            #     keywords="AI, 人工智能, 机器学习",
            # ),
        )
        return testing_site

    if isinstance(site_id, str):
        site_id = uuid.UUID(site_id)
    statement = select(Site).where(Site.id == site_id)
    result = await session.exec(statement)
    return result.one_or_none()


async def get_sites_enabled_automation(session: AsyncSession):
    statement = select(Site).where(Site.enabled_automation == True)  # noqa: E712
    result = await session.exec(statement)
    return result.all()


async def get_site_domain(session: AsyncSession, domain: str):
    """
    TODO: 优化: 可以使用 join select 的方式 一次查询搞定
    """
    statement = select(SiteHost).where(SiteHost.domain == domain)
    site_host = await session.exec(statement)
    site_host = site_host.one_or_none()

    if site_host:
        site = await get_site_by_id(session, site_host.site_id)
        return site
    return None


async def create(
    session: AsyncSession,
    item_in: SiteCreateRequest,
    user_id: uuid.UUID | str | None = None,
):
    site_to_create = Site.model_validate(item_in, update={"owner_id": user_id})
    if user_id:
        site_to_create.owner_id = user_id
    session.add(site_to_create)
    await session.commit()
    await session.refresh(site_to_create)
    await create_site_search_index(session, site_to_create, user_id)
    await session.refresh(site_to_create)
    return site_to_create



import torch
from RNNmodel import *
import time
from data_process_module import *
from torch.utils.data import DataLoader
from train_best_model import *

# 存放数据的文件夹
dataset = 'text_classify_data'

# 搜狗新闻:embedding_SougouNews.npz, 腾讯:embedding_Tencent.npz, 随机初始化:random
embedding = 'embedding_SougouNews.npz'

config = Config(dataset, embedding)
# 设置随机数种子，保证每次运行结果一致，不至于不能复现模型
np.random.seed(1)
torch.manual_seed(1)
torch.cuda.manual_seed_all(1)
torch.backends.cudnn.deterministic = True  # 保证每次结果一样

start_time = time.time()
print("Loading data...")
vocab, train_data, dev_data, test_data = get_data(config, False)
dataloaders = {
    'train': DataLoader(TextDataset(train_data, config), 128, shuffle=True),
    'dev': DataLoader(TextDataset(dev_data, config), 128, shuffle=True),
    'test': DataLoader(TextDataset(test_data, config), 128, shuffle=True)
}

time_dif = get_time_dif(start_time)
print("Time usage:", time_dif)

# train
config.n_vocab = len(vocab)
model = RNNModel(config).to(config.device)
# writer = SummaryWriter(log_dir=config.log_path + '/' + time.strftime('%m-%d_%H.%M', time.localtime()))
init_network(model)
print(model.parameters)
train_best(config, model, dataloaders)



import asyncio
import json

import structlog
from mtmai.worker import wfapp
from mtmaisdk.clients.rest.api.llm_api import LlmApi
from mtmaisdk.clients.rest.models import ScrapeGraphParams
from mtmaisdk.clients.rest_client import LogApi
from mtmaisdk.context.context import Context
from scrapegraphai.graphs import SmartScraperGraph

LOG = structlog.get_logger()


@wfapp.workflow(
    name="scrape", on_events=["scrape:run"], input_validator=ScrapeGraphParams
)
class ScrapFlow:
    @wfapp.step(timeout="20m", retries=2)
    async def graph_entry(self, hatctx: Context):
        # 获取 llm 配置
        llm_config = hatctx.rest_client.aio._api_client
        log_api = LogApi(hatctx.rest_client.aio._api_client)
        result = await log_api.log_line_list(step_run=hatctx.step_run_id)
        print(result)
        llm_api = LlmApi(hatctx.rest_client.aio._api_client)
        llm_config = await llm_api.llm_get(
            # tenant=hatctx.tenant_id,
            # slug=hatctx.node_id,
            # agent_node_run_request=hatctx.agent_node_run_request,
        )
        print(llm_config)
        # Define the configuration for the scraping pipeline
        graph_config = {
            "llm": {
                "api_key": "YOUR_OPENAI_APIKEY",
                "model": "openai/gpt-4o-mini",
            },
            "verbose": True,
            "headless": False,
        }

        # Create the SmartScraperGraph instance
        smart_scraper_graph = SmartScraperGraph(
            prompt="Extract me all the news from the website",
            source="https://www.wired.com",
            config=graph_config,
        )

        # Run the pipeline
        # result = smart_scraper_graph.run()
        result = await asyncio.to_thread(smart_scraper_graph.run)

        print(json.dumps(result, indent=4))



from textwrap import dedent

from crewai import Agent, Crew, Process, Task
from pydantic import BaseModel

# from mtmai.agents.ctx import mtmai_context
from mtmai.agents.tools.tools import get_tools
from mtmai.models.book_gen import BookOutline


async def call_crew(
    crew: Crew, input: dict | BaseModel, pydanticOutput: None | BaseModel = None
):
    if isinstance(input, BaseModel):
        input = input.model_dump()
    output = await crew.kickoff_async(inputs=input)

    if not output:
        raise ValueError("调用 crew 失败，因输出内容是空的")
    if not output.pydantic:
        # output.pydantic = GenBlogTopicsOutput.model_validate_json(
        #     repaire_json(output.raw)
        # )
        # if BaseModel:
        #     try:
        #         output.pydantic = BaseModel.model_validate_json(
        #             repaire_json(output.raw)
        #         )
        #     except Exception:
        #         raise Exception(f"call_crew 解释 JSON 出错, json: {output.raw}")
        return output.raw
    if not output.pydantic:
        raise ValueError(
            f"调用 crew 失败，llm输出不是 json 格式: \n========\n{output.raw}\n========\n"
        )
    if pydanticOutput:
        return pydanticOutput.model_validate(output.pydantic)
    return output.pydantic.model_dump()


async def crew_gen_outline(callback) -> BookOutline:
    """生成文章大纲"""
    llm = await mtmai_context.get_crawai_llm()
    researcher_agent = Agent(
        role="Research Agent",
        goal=dedent("""Gather comprehensive information about {topic} that will be used to create an organized and well-structured book outline.
Here is some additional information about the author's desired goal for the book:\n\n {goal}"""),
        backstory=dedent("""You're a seasoned researcher, known for gathering the best sources and understanding the key elements of any topic.
You aim to collect all relevant information so the book outline can be accurate and informative."""),
        tools=get_tools("search_engine"),
        llm=llm,
        verbose=True,
        max_retry_limit=100,
        max_rpm=60,
        step_callback=callback,
    )
    outliner_agent = Agent(
        role="Book Outlining Agent",
        goal=dedent("""Based on the research, generate a book outline about the following topic: {topic}
The generated outline should include all chapters in sequential order and provide a title and description for each chapter.
Here is some additional information about the author's desired goal for the book:\n\n {goal}"""),
        backstory=dedent("""You are a skilled organizer, great at turning scattered information into a structured format.
Your goal is to create clear, concise chapter outlines with all key topics and subtopics covered."""),
        llm=llm,
        verbose=True,
        step_callback=callback,
    )

    research_topic_task = Task(
        description=dedent("""Research the provided topic of {topic} to gather the most important information that will
be useful in creating a book outline. Ensure you focus on high-quality, reliable sources.

Here is some additional information about the author's desired goal for the book:\n\n {goal}
        """),
        expected_output="A set of key points and important information about {topic} that will be used to create the outline.",
        agent=researcher_agent,
    )
    generate_outline_task = Task(
        description=dedent("""Create a book outline with chapters in sequential order based on the research findings.
Ensure that each chapter has a title and a brief description that highlights the topics and subtopics to be covered.
It's important to note that each chapter is only going to be 3,000 words or less.
Also, make sure that you do not duplicate any chapters or topics in the outline.

Here is some additional information about the author's desired goal for the book:\n\n {goal}"""),
        expected_output="An outline of chapters, with titles and descriptions of what each chapter will contain. Maximum of 3 chapters.  \n\n {format_instructions}",
        output_pydantic=BookOutline,
        agent=outliner_agent,
    )
    crew = Crew(
        agents=[researcher_agent, outliner_agent],
        tasks=[research_topic_task, generate_outline_task],
        process=Process.sequential,
        verbose=True,
        step_callback=callback,
        task_callback=callback,
    )
    return crew


class GenBlogTopicsOutput(BaseModel):
    topics: list[str] = "主题列表，按优先级更好的方前面"



import jwt
from fastapi import Request
from jwt.exceptions import InvalidTokenError
from pydantic import ValidationError
from starlette.middleware.base import BaseHTTPMiddleware

from mtmai.context import set_current_user_id, set_user
from mtmai.core import security
from mtmai.core.config import settings
from mtmai.crud import curd
from mtmai.models.models import TokenPayload


class AuthMiddleware(BaseHTTPMiddleware):
    async def dispatch(self, request: Request, call_next):
        jwt_token = None
        authorization = request.headers.get("Authorization")
        if authorization:
            jwt_token = authorization[7:]  # bearer
        if not jwt_token:
            if settings.COOKIE_ACCESS_TOKEN:
                jwt_token = request.cookies.get(settings.COOKIE_ACCESS_TOKEN)

        if not jwt_token:
            return await call_next(request)
        try:
            payload = jwt.decode(
                jwt_token, settings.SECRET_KEY, algorithms=[security.ALGORITHM]
            )
            token_data = TokenPayload(**payload)
            set_current_user_id(token_data.sub)
        except (InvalidTokenError, ValidationError):
            return await call_next(request)
        user = await curd.get_user_by_id(token_data.sub)
        if user:
            set_user(user)

        return await call_next(request)



from typing import List

from pydantic import BaseModel


class ChapterOutline(BaseModel):
    title: str
    description: str


class BookOutline(BaseModel):
    chapters: List[ChapterOutline]


class Chapter(BaseModel):
    title: str
    content: str


class WriteOutlineRequest(BaseModel):
    """参数: 生成文章大纲"""

    topic: str
    goal: str


class WriteSingleChapterRequest(BaseModel):
    """参数: 生成文章一个章节"""

    goal: str
    topic: str
    chapter_title: str
    chapter_description: str
    book_outlines: list[ChapterOutline]


class GenBookState(BaseModel):
    title: str | None = (
        "The Current State of AI in September 2024: Trends Across Industries and What's Next"
    )
    book: list[Chapter] | None = []
    book_outline: list[ChapterOutline] | None = []
    topic: str | None = (
        "Exploring the latest trends in AI across different industries as of September 2024"
    )
    goal: str | None = """
        The goal of this book is to provide a comprehensive overview of the current state of artificial intelligence in September 2024.
        It will delve into the latest trends impacting various industries, analyze significant advancements,
        and discuss potential future developments. The book aims to inform readers about cutting-edge AI technologies
        and prepare them for upcoming innovations in the field.
    """


# class BlogState(BaseModel):
#     description: str = Field(description="博客功能定位介绍")
#     seo_keywords: str = Field(description="SEO关键词列表")
#     day_published_count: int = Field(description="已完成的日更数量")
#     day_publish_count_hint: int = Field(description="建议日更数量")


# class BlogDescription(BaseModel):
#     """博客网站描述"""

#     description: str



# coding: utf-8

# flake8: noqa
"""
    Mtmai API

    The Mtmai API

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from mtmai.gomtmclients.rest.models.api_error import APIError
from mtmai.gomtmclients.rest.models.api_errors import APIErrors
from mtmai.gomtmclients.rest.models.api_meta import APIMeta
from mtmai.gomtmclients.rest.models.api_meta_auth import APIMetaAuth
from mtmai.gomtmclients.rest.models.api_meta_integration import APIMetaIntegration
from mtmai.gomtmclients.rest.models.api_meta_posthog import APIMetaPosthog
from mtmai.gomtmclients.rest.models.api_resource_meta import APIResourceMeta
from mtmai.gomtmclients.rest.models.api_token import APIToken
from mtmai.gomtmclients.rest.models.accept_invite_request import AcceptInviteRequest
from mtmai.gomtmclients.rest.models.agent import Agent
from mtmai.gomtmclients.rest.models.agent_action import AgentAction
from mtmai.gomtmclients.rest.models.agent_config import AgentConfig
from mtmai.gomtmclients.rest.models.agent_finish import AgentFinish
from mtmai.gomtmclients.rest.models.agent_message_config import AgentMessageConfig
from mtmai.gomtmclients.rest.models.agent_node import AgentNode
from mtmai.gomtmclients.rest.models.agent_node_create_request import AgentNodeCreateRequest
from mtmai.gomtmclients.rest.models.agent_node_input import AgentNodeInput
from mtmai.gomtmclients.rest.models.agent_node_list import AgentNodeList
from mtmai.gomtmclients.rest.models.agent_node_output import AgentNodeOutput
from mtmai.gomtmclients.rest.models.agent_node_run import AgentNodeRun
from mtmai.gomtmclients.rest.models.agent_node_run200_response import AgentNodeRun200Response
from mtmai.gomtmclients.rest.models.agent_node_run_input import AgentNodeRunInput
from mtmai.gomtmclients.rest.models.agent_node_run_input_params import AgentNodeRunInputParams
from mtmai.gomtmclients.rest.models.agent_node_update_request import AgentNodeUpdateRequest
from mtmai.gomtmclients.rest.models.agent_run_form import AgentRunForm
from mtmai.gomtmclients.rest.models.agent_state import AgentState
from mtmai.gomtmclients.rest.models.agent_step import AgentStep
from mtmai.gomtmclients.rest.models.agent_task_step import AgentTaskStep
from mtmai.gomtmclients.rest.models.agent_task_tool import AgentTaskTool
from mtmai.gomtmclients.rest.models.agent_types import AgentTypes
from mtmai.gomtmclients.rest.models.artifact import Artifact
from mtmai.gomtmclients.rest.models.artifact_code_v3 import ArtifactCodeV3
from mtmai.gomtmclients.rest.models.artifact_length_options import ArtifactLengthOptions
from mtmai.gomtmclients.rest.models.artifact_list import ArtifactList
from mtmai.gomtmclients.rest.models.artifact_markdown_v3 import ArtifactMarkdownV3
from mtmai.gomtmclients.rest.models.artifact_tool_response import ArtifactToolResponse
from mtmai.gomtmclients.rest.models.artifact_v3 import ArtifactV3
from mtmai.gomtmclients.rest.models.artifact_v3_contents_inner import ArtifactV3ContentsInner
from mtmai.gomtmclients.rest.models.assisant import Assisant
from mtmai.gomtmclients.rest.models.assisant_config import AssisantConfig
from mtmai.gomtmclients.rest.models.assisant_config_configurable import AssisantConfigConfigurable
from mtmai.gomtmclients.rest.models.assisant_list import AssisantList
from mtmai.gomtmclients.rest.models.assisant_state import AssisantState
from mtmai.gomtmclients.rest.models.assistant import Assistant
from mtmai.gomtmclients.rest.models.assistant_agent_config import AssistantAgentConfig
from mtmai.gomtmclients.rest.models.assistant_base import AssistantBase
from mtmai.gomtmclients.rest.models.azure_open_ai_model_config import AzureOpenAIModelConfig
from mtmai.gomtmclients.rest.models.base_agent_config import BaseAgentConfig
from mtmai.gomtmclients.rest.models.base_config import BaseConfig
from mtmai.gomtmclients.rest.models.base_message_config import BaseMessageConfig
from mtmai.gomtmclients.rest.models.base_model_config import BaseModelConfig
from mtmai.gomtmclients.rest.models.base_state import BaseState
from mtmai.gomtmclients.rest.models.base_team_config import BaseTeamConfig
from mtmai.gomtmclients.rest.models.base_termination_config import BaseTerminationConfig
from mtmai.gomtmclients.rest.models.blog import Blog
from mtmai.gomtmclients.rest.models.blog_config import BlogConfig
from mtmai.gomtmclients.rest.models.blog_gen_config import BlogGenConfig
from mtmai.gomtmclients.rest.models.blog_list import BlogList
from mtmai.gomtmclients.rest.models.blog_post import BlogPost
from mtmai.gomtmclients.rest.models.blog_post_list import BlogPostList
from mtmai.gomtmclients.rest.models.blog_post_state import BlogPostState
from mtmai.gomtmclients.rest.models.blog_post_state_outlines_inner import BlogPostStateOutlinesInner
from mtmai.gomtmclients.rest.models.blog_task_state import BlogTaskState
from mtmai.gomtmclients.rest.models.browser import Browser
from mtmai.gomtmclients.rest.models.browser_list import BrowserList
from mtmai.gomtmclients.rest.models.browser_params import BrowserParams
from mtmai.gomtmclients.rest.models.browser_update import BrowserUpdate
from mtmai.gomtmclients.rest.models.bulk_create_event_request import BulkCreateEventRequest
from mtmai.gomtmclients.rest.models.bulk_create_event_response import BulkCreateEventResponse
from mtmai.gomtmclients.rest.models.call_agent import CallAgent
from mtmai.gomtmclients.rest.models.call_agent_result import CallAgentResult
from mtmai.gomtmclients.rest.models.cancel_event_request import CancelEventRequest
from mtmai.gomtmclients.rest.models.canvas_graph_params import CanvasGraphParams
from mtmai.gomtmclients.rest.models.chat_completions_req import ChatCompletionsReq
from mtmai.gomtmclients.rest.models.chat_history_list import ChatHistoryList
from mtmai.gomtmclients.rest.models.chat_message import ChatMessage
from mtmai.gomtmclients.rest.models.chat_message_config import ChatMessageConfig
from mtmai.gomtmclients.rest.models.chat_message_role import ChatMessageRole
from mtmai.gomtmclients.rest.models.chat_messages import ChatMessages
from mtmai.gomtmclients.rest.models.chat_model import ChatModel
from mtmai.gomtmclients.rest.models.chat_model_list import ChatModelList
from mtmai.gomtmclients.rest.models.chat_req import ChatReq
from mtmai.gomtmclients.rest.models.code_highlight import CodeHighlight
from mtmai.gomtmclients.rest.models.combination_termination_config import CombinationTerminationConfig
from mtmai.gomtmclients.rest.models.common_result import CommonResult
from mtmai.gomtmclients.rest.models.component_model import ComponentModel
from mtmai.gomtmclients.rest.models.create_api_token_request import CreateAPITokenRequest
from mtmai.gomtmclients.rest.models.create_api_token_response import CreateAPITokenResponse
from mtmai.gomtmclients.rest.models.create_artifactt_request import CreateArtifacttRequest
from mtmai.gomtmclients.rest.models.create_blog_post_request import CreateBlogPostRequest
from mtmai.gomtmclients.rest.models.create_blog_request import CreateBlogRequest
from mtmai.gomtmclients.rest.models.create_event_request import CreateEventRequest
from mtmai.gomtmclients.rest.models.create_post_request import CreatePostRequest
from mtmai.gomtmclients.rest.models.create_pull_request_from_step_run import CreatePullRequestFromStepRun
from mtmai.gomtmclients.rest.models.create_sns_integration_request import CreateSNSIntegrationRequest
from mtmai.gomtmclients.rest.models.create_site_host_request import CreateSiteHostRequest
from mtmai.gomtmclients.rest.models.create_site_request import CreateSiteRequest
from mtmai.gomtmclients.rest.models.create_tenant_alert_email_group_request import CreateTenantAlertEmailGroupRequest
from mtmai.gomtmclients.rest.models.create_tenant_invite_request import CreateTenantInviteRequest
from mtmai.gomtmclients.rest.models.create_tenant_request import CreateTenantRequest
from mtmai.gomtmclients.rest.models.crew_ai_params import CrewAIParams
from mtmai.gomtmclients.rest.models.crew_ai_agent import CrewAiAgent
from mtmai.gomtmclients.rest.models.crew_ai_task import CrewAiTask
from mtmai.gomtmclients.rest.models.cron_workflows import CronWorkflows
from mtmai.gomtmclients.rest.models.cron_workflows_list import CronWorkflowsList
from mtmai.gomtmclients.rest.models.cron_workflows_order_by_field import CronWorkflowsOrderByField
from mtmai.gomtmclients.rest.models.custom_quick_action import CustomQuickAction
from mtmai.gomtmclients.rest.models.db_model import DBModel
from mtmai.gomtmclients.rest.models.dash_sidebar_item import DashSidebarItem
from mtmai.gomtmclients.rest.models.endpoint import Endpoint
from mtmai.gomtmclients.rest.models.endpoint_list import EndpointList
from mtmai.gomtmclients.rest.models.env import Env
from mtmai.gomtmclients.rest.models.env_list import EnvList
from mtmai.gomtmclients.rest.models.event import Event
from mtmai.gomtmclients.rest.models.event_data import EventData
from mtmai.gomtmclients.rest.models.event_key_list import EventKeyList
from mtmai.gomtmclients.rest.models.event_list import EventList
from mtmai.gomtmclients.rest.models.event_order_by_direction import EventOrderByDirection
from mtmai.gomtmclients.rest.models.event_order_by_field import EventOrderByField
from mtmai.gomtmclients.rest.models.event_update_cancel200_response import EventUpdateCancel200Response
from mtmai.gomtmclients.rest.models.event_workflow_run_summary import EventWorkflowRunSummary
from mtmai.gomtmclients.rest.models.evt_node_step import EvtNodeStep
from mtmai.gomtmclients.rest.models.file_surfer_agent_config import FileSurferAgentConfig
from mtmai.gomtmclients.rest.models.flow_names import FlowNames
from mtmai.gomtmclients.rest.models.form_field import FormField
from mtmai.gomtmclients.rest.models.frontend_config import FrontendConfig
from mtmai.gomtmclients.rest.models.function_call import FunctionCall
from mtmai.gomtmclients.rest.models.function_execution_result import FunctionExecutionResult
from mtmai.gomtmclients.rest.models.gallery import Gallery
from mtmai.gomtmclients.rest.models.gallery_components import GalleryComponents
from mtmai.gomtmclients.rest.models.gallery_items import GalleryItems
from mtmai.gomtmclients.rest.models.gallery_list import GalleryList
from mtmai.gomtmclients.rest.models.gallery_meta import GalleryMeta
from mtmai.gomtmclients.rest.models.gallery_update import GalleryUpdate
from mtmai.gomtmclients.rest.models.gen_article_input import GenArticleInput
from mtmai.gomtmclients.rest.models.gen_article_state import GenArticleState
from mtmai.gomtmclients.rest.models.gen_article_state_all_of_oulines import GenArticleStateAllOfOulines
from mtmai.gomtmclients.rest.models.gen_topic_result import GenTopicResult
from mtmai.gomtmclients.rest.models.get_step_run_diff_response import GetStepRunDiffResponse
from mtmai.gomtmclients.rest.models.handoff_message_config import HandoffMessageConfig
from mtmai.gomtmclients.rest.models.hf_account import HfAccount
from mtmai.gomtmclients.rest.models.image_content import ImageContent
from mtmai.gomtmclients.rest.models.inner_message_config import InnerMessageConfig
from mtmai.gomtmclients.rest.models.job import Job
from mtmai.gomtmclients.rest.models.job_run import JobRun
from mtmai.gomtmclients.rest.models.job_run_status import JobRunStatus
from mtmai.gomtmclients.rest.models.language_options import LanguageOptions
from mtmai.gomtmclients.rest.models.list_api_tokens_response import ListAPITokensResponse
from mtmai.gomtmclients.rest.models.list_pull_requests_response import ListPullRequestsResponse
from mtmai.gomtmclients.rest.models.list_sns_integrations import ListSNSIntegrations
from mtmai.gomtmclients.rest.models.list_slack_webhooks import ListSlackWebhooks
from mtmai.gomtmclients.rest.models.llm_config import LlmConfig
from mtmai.gomtmclients.rest.models.log_line import LogLine
from mtmai.gomtmclients.rest.models.log_line_level import LogLineLevel
from mtmai.gomtmclients.rest.models.log_line_list import LogLineList
from mtmai.gomtmclients.rest.models.log_line_order_by_direction import LogLineOrderByDirection
from mtmai.gomtmclients.rest.models.log_line_order_by_field import LogLineOrderByField
from mtmai.gomtmclients.rest.models.magentic_one_coder_agent_config import MagenticOneCoderAgentConfig
from mtmai.gomtmclients.rest.models.max_message_termination_config import MaxMessageTerminationConfig
from mtmai.gomtmclients.rest.models.message_v2 import MessageV2
from mtmai.gomtmclients.rest.models.model_config import ModelConfig
from mtmai.gomtmclients.rest.models.model_config_azure_config import ModelConfigAzureConfig
from mtmai.gomtmclients.rest.models.model_config_v2 import ModelConfigV2
from mtmai.gomtmclients.rest.models.model_info import ModelInfo
from mtmai.gomtmclients.rest.models.model_types import ModelTypes
from mtmai.gomtmclients.rest.models.mtmai_worker_config200_response import MtmaiWorkerConfig200Response
from mtmai.gomtmclients.rest.models.multi_modal_message_config import MultiModalMessageConfig
from mtmai.gomtmclients.rest.models.multi_modal_message_config_all_of_content import MultiModalMessageConfigAllOfContent
from mtmai.gomtmclients.rest.models.multimodal_web_surfer_agent_config import MultimodalWebSurferAgentConfig
from mtmai.gomtmclients.rest.models.node_run_action import NodeRunAction
from mtmai.gomtmclients.rest.models.open_ai_model_config import OpenAIModelConfig
from mtmai.gomtmclients.rest.models.operataion_guide_response import OperataionGuideResponse
from mtmai.gomtmclients.rest.models.operation_enum import OperationEnum
from mtmai.gomtmclients.rest.models.outline import Outline
from mtmai.gomtmclients.rest.models.pagination_response import PaginationResponse
from mtmai.gomtmclients.rest.models.platform import Platform
from mtmai.gomtmclients.rest.models.platform_account import PlatformAccount
from mtmai.gomtmclients.rest.models.platform_account_list import PlatformAccountList
from mtmai.gomtmclients.rest.models.platform_account_update import PlatformAccountUpdate
from mtmai.gomtmclients.rest.models.platform_list import PlatformList
from mtmai.gomtmclients.rest.models.platform_update import PlatformUpdate
from mtmai.gomtmclients.rest.models.post import Post
from mtmai.gomtmclients.rest.models.post_list import PostList
from mtmai.gomtmclients.rest.models.postiz_channel import PostizChannel
from mtmai.gomtmclients.rest.models.postiz_state import PostizState
from mtmai.gomtmclients.rest.models.programming_language_options import ProgrammingLanguageOptions
from mtmai.gomtmclients.rest.models.prompt import Prompt
from mtmai.gomtmclients.rest.models.prompt_list import PromptList
from mtmai.gomtmclients.rest.models.proxy import Proxy
from mtmai.gomtmclients.rest.models.proxy_list import ProxyList
from mtmai.gomtmclients.rest.models.proxy_update import ProxyUpdate
from mtmai.gomtmclients.rest.models.pull_request import PullRequest
from mtmai.gomtmclients.rest.models.pull_request_state import PullRequestState
from mtmai.gomtmclients.rest.models.queue_metrics import QueueMetrics
from mtmai.gomtmclients.rest.models.rate_limit import RateLimit
from mtmai.gomtmclients.rest.models.rate_limit_list import RateLimitList
from mtmai.gomtmclients.rest.models.rate_limit_order_by_direction import RateLimitOrderByDirection
from mtmai.gomtmclients.rest.models.rate_limit_order_by_field import RateLimitOrderByField
from mtmai.gomtmclients.rest.models.reading_level_options import ReadingLevelOptions
from mtmai.gomtmclients.rest.models.recent_step_runs import RecentStepRuns
from mtmai.gomtmclients.rest.models.reflections import Reflections
from mtmai.gomtmclients.rest.models.reject_invite_request import RejectInviteRequest
from mtmai.gomtmclients.rest.models.replay_event_request import ReplayEventRequest
from mtmai.gomtmclients.rest.models.replay_workflow_runs_request import ReplayWorkflowRunsRequest
from mtmai.gomtmclients.rest.models.replay_workflow_runs_response import ReplayWorkflowRunsResponse
from mtmai.gomtmclients.rest.models.request_usage import RequestUsage
from mtmai.gomtmclients.rest.models.rerun_step_run_request import RerunStepRunRequest
from mtmai.gomtmclients.rest.models.research_request import ResearchRequest
from mtmai.gomtmclients.rest.models.research_response import ResearchResponse
from mtmai.gomtmclients.rest.models.rewrite_artifact_meta_tool_response import RewriteArtifactMetaToolResponse
from mtmai.gomtmclients.rest.models.rewrite_artifact_meta_tool_response_one_of import RewriteArtifactMetaToolResponseOneOf
from mtmai.gomtmclients.rest.models.rewrite_artifact_meta_tool_response_one_of1 import RewriteArtifactMetaToolResponseOneOf1
from mtmai.gomtmclients.rest.models.round_robin_group_chat_config import RoundRobinGroupChatConfig
from mtmai.gomtmclients.rest.models.run import Run
from mtmai.gomtmclients.rest.models.run_new_task_response import RunNewTaskResponse
from mtmai.gomtmclients.rest.models.run_status import RunStatus
from mtmai.gomtmclients.rest.models.sns_integration import SNSIntegration
from mtmai.gomtmclients.rest.models.scheduled_run_status import ScheduledRunStatus
from mtmai.gomtmclients.rest.models.scheduled_workflows import ScheduledWorkflows
from mtmai.gomtmclients.rest.models.scheduled_workflows_list import ScheduledWorkflowsList
from mtmai.gomtmclients.rest.models.scheduled_workflows_order_by_field import ScheduledWorkflowsOrderByField
from mtmai.gomtmclients.rest.models.schema_form import SchemaForm
from mtmai.gomtmclients.rest.models.scrape_graph_params import ScrapeGraphParams
from mtmai.gomtmclients.rest.models.section import Section
from mtmai.gomtmclients.rest.models.selector_group_chat_config import SelectorGroupChatConfig
from mtmai.gomtmclients.rest.models.semaphore_slots import SemaphoreSlots
from mtmai.gomtmclients.rest.models.session import Session
from mtmai.gomtmclients.rest.models.session_list import SessionList
from mtmai.gomtmclients.rest.models.session_runs import SessionRuns
from mtmai.gomtmclients.rest.models.session_update import SessionUpdate
from mtmai.gomtmclients.rest.models.siderbar_config import SiderbarConfig
from mtmai.gomtmclients.rest.models.site import Site
from mtmai.gomtmclients.rest.models.site_host import SiteHost
from mtmai.gomtmclients.rest.models.site_host_list import SiteHostList
from mtmai.gomtmclients.rest.models.site_list import SiteList
from mtmai.gomtmclients.rest.models.slack_webhook import SlackWebhook
from mtmai.gomtmclients.rest.models.step import Step
from mtmai.gomtmclients.rest.models.step_run import StepRun
from mtmai.gomtmclients.rest.models.step_run_archive import StepRunArchive
from mtmai.gomtmclients.rest.models.step_run_archive_list import StepRunArchiveList
from mtmai.gomtmclients.rest.models.step_run_diff import StepRunDiff
from mtmai.gomtmclients.rest.models.step_run_event import StepRunEvent
from mtmai.gomtmclients.rest.models.step_run_event_list import StepRunEventList
from mtmai.gomtmclients.rest.models.step_run_event_reason import StepRunEventReason
from mtmai.gomtmclients.rest.models.step_run_event_severity import StepRunEventSeverity
from mtmai.gomtmclients.rest.models.step_run_status import StepRunStatus
from mtmai.gomtmclients.rest.models.stop_message_config import StopMessageConfig
from mtmai.gomtmclients.rest.models.subscribe_request import SubscribeRequest
from mtmai.gomtmclients.rest.models.subsection import Subsection
from mtmai.gomtmclients.rest.models.task_result import TaskResult
from mtmai.gomtmclients.rest.models.team import Team
from mtmai.gomtmclients.rest.models.team_config import TeamConfig
from mtmai.gomtmclients.rest.models.team_list import TeamList
from mtmai.gomtmclients.rest.models.team_result import TeamResult
from mtmai.gomtmclients.rest.models.team_types import TeamTypes
from mtmai.gomtmclients.rest.models.team_update import TeamUpdate
from mtmai.gomtmclients.rest.models.tenant import Tenant
from mtmai.gomtmclients.rest.models.tenant_alert_email_group import TenantAlertEmailGroup
from mtmai.gomtmclients.rest.models.tenant_alert_email_group_list import TenantAlertEmailGroupList
from mtmai.gomtmclients.rest.models.tenant_alerting_settings import TenantAlertingSettings
from mtmai.gomtmclients.rest.models.tenant_invite import TenantInvite
from mtmai.gomtmclients.rest.models.tenant_invite_list import TenantInviteList
from mtmai.gomtmclients.rest.models.tenant_list import TenantList
from mtmai.gomtmclients.rest.models.tenant_member import TenantMember
from mtmai.gomtmclients.rest.models.tenant_member_list import TenantMemberList
from mtmai.gomtmclients.rest.models.tenant_member_role import TenantMemberRole
from mtmai.gomtmclients.rest.models.tenant_queue_metrics import TenantQueueMetrics
from mtmai.gomtmclients.rest.models.tenant_resource import TenantResource
from mtmai.gomtmclients.rest.models.tenant_resource_limit import TenantResourceLimit
from mtmai.gomtmclients.rest.models.tenant_resource_policy import TenantResourcePolicy
from mtmai.gomtmclients.rest.models.tenant_step_run_queue_metrics import TenantStepRunQueueMetrics
from mtmai.gomtmclients.rest.models.termination_config import TerminationConfig
from mtmai.gomtmclients.rest.models.termination_types import TerminationTypes
from mtmai.gomtmclients.rest.models.text_highlight import TextHighlight
from mtmai.gomtmclients.rest.models.text_mention_termination_config import TextMentionTerminationConfig
from mtmai.gomtmclients.rest.models.text_message_config import TextMessageConfig
from mtmai.gomtmclients.rest.models.token_chunk import TokenChunk
from mtmai.gomtmclients.rest.models.tool_call_message_config import ToolCallMessageConfig
from mtmai.gomtmclients.rest.models.tool_call_result import ToolCallResult
from mtmai.gomtmclients.rest.models.tool_call_result_message_config import ToolCallResultMessageConfig
from mtmai.gomtmclients.rest.models.tool_config import ToolConfig
from mtmai.gomtmclients.rest.models.tool_types import ToolTypes
from mtmai.gomtmclients.rest.models.trigger_workflow_run_request import TriggerWorkflowRunRequest
from mtmai.gomtmclients.rest.models.update_blog_request import UpdateBlogRequest
from mtmai.gomtmclients.rest.models.update_endpoint_request import UpdateEndpointRequest
from mtmai.gomtmclients.rest.models.update_post_request import UpdatePostRequest
from mtmai.gomtmclients.rest.models.update_site_request import UpdateSiteRequest
from mtmai.gomtmclients.rest.models.update_tenant_alert_email_group_request import UpdateTenantAlertEmailGroupRequest
from mtmai.gomtmclients.rest.models.update_tenant_invite_request import UpdateTenantInviteRequest
from mtmai.gomtmclients.rest.models.update_tenant_request import UpdateTenantRequest
from mtmai.gomtmclients.rest.models.update_worker_request import UpdateWorkerRequest
from mtmai.gomtmclients.rest.models.user import User
from mtmai.gomtmclients.rest.models.user_change_password_request import UserChangePasswordRequest
from mtmai.gomtmclients.rest.models.user_login_request import UserLoginRequest
from mtmai.gomtmclients.rest.models.user_proxy_agent_config import UserProxyAgentConfig
from mtmai.gomtmclients.rest.models.user_register_request import UserRegisterRequest
from mtmai.gomtmclients.rest.models.user_tenant_memberships_list import UserTenantMembershipsList
from mtmai.gomtmclients.rest.models.user_tenant_public import UserTenantPublic
from mtmai.gomtmclients.rest.models.web_search_result import WebSearchResult
from mtmai.gomtmclients.rest.models.web_socket_message import WebSocketMessage
from mtmai.gomtmclients.rest.models.webhook_worker import WebhookWorker
from mtmai.gomtmclients.rest.models.webhook_worker_create_request import WebhookWorkerCreateRequest
from mtmai.gomtmclients.rest.models.webhook_worker_create_response import WebhookWorkerCreateResponse
from mtmai.gomtmclients.rest.models.webhook_worker_created import WebhookWorkerCreated
from mtmai.gomtmclients.rest.models.webhook_worker_list_response import WebhookWorkerListResponse
from mtmai.gomtmclients.rest.models.webhook_worker_request import WebhookWorkerRequest
from mtmai.gomtmclients.rest.models.webhook_worker_request_list_response import WebhookWorkerRequestListResponse
from mtmai.gomtmclients.rest.models.webhook_worker_request_method import WebhookWorkerRequestMethod
from mtmai.gomtmclients.rest.models.worker import Worker
from mtmai.gomtmclients.rest.models.worker_config import WorkerConfig
from mtmai.gomtmclients.rest.models.worker_label import WorkerLabel
from mtmai.gomtmclients.rest.models.worker_list import WorkerList
from mtmai.gomtmclients.rest.models.worker_runtime_info import WorkerRuntimeInfo
from mtmai.gomtmclients.rest.models.worker_runtime_sdks import WorkerRuntimeSDKs
from mtmai.gomtmclients.rest.models.workflow import Workflow
from mtmai.gomtmclients.rest.models.workflow_concurrency import WorkflowConcurrency
from mtmai.gomtmclients.rest.models.workflow_kind import WorkflowKind
from mtmai.gomtmclients.rest.models.workflow_list import WorkflowList
from mtmai.gomtmclients.rest.models.workflow_metrics import WorkflowMetrics
from mtmai.gomtmclients.rest.models.workflow_run import WorkflowRun
from mtmai.gomtmclients.rest.models.workflow_run_list import WorkflowRunList
from mtmai.gomtmclients.rest.models.workflow_run_order_by_direction import WorkflowRunOrderByDirection
from mtmai.gomtmclients.rest.models.workflow_run_order_by_field import WorkflowRunOrderByField
from mtmai.gomtmclients.rest.models.workflow_run_shape import WorkflowRunShape
from mtmai.gomtmclients.rest.models.workflow_run_status import WorkflowRunStatus
from mtmai.gomtmclients.rest.models.workflow_run_triggered_by import WorkflowRunTriggeredBy
from mtmai.gomtmclients.rest.models.workflow_runs_cancel_request import WorkflowRunsCancelRequest
from mtmai.gomtmclients.rest.models.workflow_runs_metrics import WorkflowRunsMetrics
from mtmai.gomtmclients.rest.models.workflow_runs_metrics_counts import WorkflowRunsMetricsCounts
from mtmai.gomtmclients.rest.models.workflow_tag import WorkflowTag
from mtmai.gomtmclients.rest.models.workflow_trigger_cron_ref import WorkflowTriggerCronRef
from mtmai.gomtmclients.rest.models.workflow_trigger_event_ref import WorkflowTriggerEventRef
from mtmai.gomtmclients.rest.models.workflow_triggers import WorkflowTriggers
from mtmai.gomtmclients.rest.models.workflow_update_request import WorkflowUpdateRequest
from mtmai.gomtmclients.rest.models.workflow_version import WorkflowVersion
from mtmai.gomtmclients.rest.models.workflow_version_definition import WorkflowVersionDefinition
from mtmai.gomtmclients.rest.models.workflow_version_meta import WorkflowVersionMeta
from mtmai.gomtmclients.rest.models.workflow_workers_count import WorkflowWorkersCount



import torch.nn as nn
import time
import torch
import copy
import pandas as pd
import datetime
from sklearn import metrics
import numpy as np


# 权重初始化，默认xavier
def init_network(model, method='xavier', exclude='embedding', seed=123):
    for name, w in model.named_parameters():
        if exclude not in name:
            if 'weight' in name:
                if method == 'xavier':
                    nn.init.xavier_normal_(w)
                elif method == 'kaiming':
                    nn.init.kaiming_normal_(w)
                else:
                    nn.init.normal_(w)
            elif 'bias' in name:
                nn.init.constant_(w, 0)
            else:
                pass


def train_best(config, model, dataloaders, log_step=100):
    '''
    训练模型
    :param config: 超参数
    :param model: 模型
    :param dataloaders: 处理后的数据，包含trian,dev,test
    :param log_step: 每隔多少个batch打印一次数据，默认100
    :return: 训练的指标
    '''

    optimizer = torch.optim.Adam(model.parameters(), lr=config.learning_rate)
    loss_function = torch.nn.CrossEntropyLoss()
    best_acc = 0
    # 最优模型
    best_model = copy.deepcopy(model.state_dict())

    total_step = 0  # 记录进行到多少batch
    dev_best_loss = float('inf')
    last_improve = 0  # 记录上次验证集loss下降的batch数
    flag = False  # 记录是否很久没有效果提升

    # 保存每一个epoch的信息
    dfhistory = pd.DataFrame(columns=["epoch", "train_loss", "train_acc", "dev_loss", "dev_acc"])

    device = config.device

    print("Start Training...\n")
    nowtime = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
    print("==========" * 8 + "%s\n" % nowtime)

    for i in range(config.num_epochs):

        # 1，训练循环----------------------------------------------------------------


        # 将数据全部取完

        # 记录每一个batch
        step = 0

        print('Epoch [{}/{}]\n'.format(i + 1, config.num_epochs))

        for inputs, labels in dataloaders['train']:
            # 训练模式，可以更新参数
            model.train()

            inputs = inputs.to(device)
            labels = labels.to(device)
            # 梯度清零，防止累加
            optimizer.zero_grad()

            outputs = model(inputs)
            loss = loss_function(outputs, labels)
            loss.backward()
            optimizer.step()

            total_step += 1
            step += 1

            if step % log_step == 0:
                true = labels.data.cpu()
                predic = torch.max(outputs.data, 1)[1].cpu()
                train_loss = loss.item()
                train_acc = metrics.accuracy_score(true, predic)

                # 2，开发集验证----------------------------------------------------------------
                dev_acc, dev_loss = dev_eval(model, dataloaders['dev'], loss_function)

                dfhistory.loc[i] = (i, train_loss, train_acc, dev_loss, dev_acc)

                if dev_loss < dev_best_loss:
                    dev_best_loss = dev_loss
                    torch.save(model.state_dict(), config.save_path)
                    last_improve = total_step

                print("[step = {} batch]  train_loss = {:.3f}, train_acc = {:.2%}, dev_loss = {:.3f}, dev_acc = {:.2%}".
                      format(step, train_loss, train_acc, dev_loss, dev_acc))

            if total_step - last_improve > config.require_improvement:
                # 验证集loss超过1000batch没下降，结束训练
                print("No optimization for a long time, auto-stopping...")
                flag = True
                break
        if flag:
            break

    # 3，验证循环----------------------------------------------------------------
    model.load_state_dict(torch.load(config.save_path))
    model.eval()
    start_time = time.time()
    test_acc, test_loss = dev_eval(model, dataloaders['test'], loss_function)
    print('================'*8)
    print('test_loss: {:.3f}      test_acc: {:.2%}'.format(test_loss, test_acc))

    return dfhistory


def dev_eval(model, data, loss_function):
    '''
    得到开发集和测试集的准确率和loss
    :param model: 模型
    :param data: 测试集集和开发集的数据
    :param loss_function: 损失函数
    :return: 损失和准确率
    '''
    model.eval()
    loss_total = 0
    predict_all = np.array([], dtype=int)
    labels_all = np.array([], dtype=int)
    with torch.no_grad():
        for texts, labels in data:
            outputs = model(texts)
            loss = loss_function(outputs, labels)
            loss_total += loss.item()
            labels = labels.data.cpu().numpy()
            predic = torch.max(outputs.data, 1)[1].cpu().numpy()
            labels_all = np.append(labels_all, labels)
            predict_all = np.append(predict_all, predic)

    acc = metrics.accuracy_score(labels_all, predict_all)
    return acc, loss_total / len(data)



# 工具库
import base64
import csv
import datetime
import json
import os
import re
import smtplib
import sys
import time
import uuid
from email.header import Header
from email.mime.text import MIMEText
from urllib.parse import urlparse

import pymysql

# import pandas as pd
# import xlsxwriter
import requests
from bs4 import BeautifulSoup

# import keyboard
from openpyxl import Workbook, load_workbook


def send_email(config):
    """
    发送邮件的函数。

    :param config: 包含邮件配置信息的字典。
    """
    # 校验配置信息是否完整
    # required_keys = ["host", "port", "username", "password", "from", "to", "subject", "content"]
    # missing_keys = [key for key in required_keys if key not in config]
    # if missing_keys:
    #     raise ValueError(f"邮件配置缺少必要的键: {', '.join(missing_keys)}")
    try:
        print("正在发送邮件到：" + config["to"])
        message = MIMEText(config["content"], "plain", "utf-8")
        message["From"] = Header(
            f"{config['username'].split('@')[0]} <{config['username']}>"
        )
        to_name_list = []
        for address in config["to"].split(","):
            address = address.strip()
            name = address.split("@")[0]
            to_name_list.append(f"{name} <{address}>")
        to_name_list = ", ".join(to_name_list)
        message["To"] = Header(to_name_list)
        message["Subject"] = Header(config["subject"], "utf-8")
        # 使用SSL加密方式连接邮件服务器
        smtp_server = smtplib.SMTP_SSL(config["host"], config["port"])
        smtp_server.login(config["username"], config["password"])
        to_address_list = config["to"].split(",")
        smtp_server.sendmail(config["username"], to_address_list, message.as_string())
        print("邮件发送成功|Email sent successfully")
    except Exception as e:
        print(f"无法发送邮件，发生错误：{e}")
        print(f"Failed to send email, error: {e}")
    finally:
        try:
            smtp_server.quit()
        except:
            pass


def rename_downloaded_file(download_dir, stop_event):
    original_files = set(os.listdir(download_dir))

    while not stop_event.is_set():
        files = os.listdir(download_dir)
        for file in files:
            if file in original_files:
                continue  # 跳过原始文件和已重命名的文件

            full_path = os.path.join(download_dir, file)

            if (
                not full_path.endswith(".crdownload")
                and not full_path.endswith(".htm")
                and not full_path.endswith(".html")
                and not full_path.startswith("esfile_")
            ):
                new_name = (
                    "esfile_"
                    + file.split("/")[-1]
                    + "_"
                    + str(uuid.uuid4())
                    + "_"
                    + file.split("/")[-1]
                )
                new_path = os.path.join(download_dir, new_name)
                try:
                    os.rename(full_path, new_path)
                    original_files.add(new_name)  # 记录新文件名以避免再次重命名
                    print(f"文件已重命名为|File has been renamed to: {new_path}")
                except:
                    print("文件重命名失败|File rename failed")

        time.sleep(1)  # 每一秒检查一次
        # print("下载文件重命名监控中，请等待...|Download file rename monitoring, please wait...")
    print("下载文件重命名监控已停止。|Download file rename monitoring has stopped.")


def is_valid_url(url):
    try:
        result = urlparse(url)
        return all([result.scheme, result.netloc])
    except ValueError:
        return False


def lowercase_tags_in_xpath(xpath):
    return re.sub(r"([A-Z]+)(?=[\[\]//]|$)", lambda x: x.group(0).lower(), xpath)


# 提取HTML中的文本内容
def extract_text_from_html(html_content):
    soup = BeautifulSoup(html_content, "lxml")  # 使用lxml作为解析器
    for script in soup(["script", "style"]):  # 去除脚本和样式内容
        script.extract()
    for p_tag in soup.find_all("p"):
        p_tag.append(soup.new_tag("br"))  # 在每个p标签后添加br标签
        p_tag.append("\n")  # 在每个p标签后添加换行符
    text = soup.get_text()
    return text


# 将文本按照行分割并去除额外空白
def split_text_by_lines(text):
    lines = text.splitlines()
    lines = [line.strip() for line in lines if line.strip()]  # 去除空行和首尾空格
    return "\n".join(lines)


def on_press_creator(press_time, event):
    def on_press(key):
        try:
            if key.char == press_time["pause_key"]:
                if press_time["is_pressed"] == False:  # 没按下p键时，记录按下p键的时间
                    press_time["duration"] = time.time()
                    press_time["is_pressed"] = True
                else:  # 按下p键时，判断按下p键的时间是否超过2.5秒
                    duration = time.time() - press_time["duration"]
                    if duration > 2:
                        if event._flag == False:
                            print(
                                "任务执行中，长按"
                                + press_time["pause_key"]
                                + "键暂停执行。"
                            )
                            print(
                                "Task is running, long press '"
                                + press_time["pause_key"]
                                + "' to pause."
                            )
                            # 设置Event的值为True，使得线程b可以继续执行
                            event.set()
                        else:
                            # 设置Event的值为False，使得线程b暂停执行
                            print(
                                "任务已暂停，长按"
                                + press_time["pause_key"]
                                + "键继续执行..."
                            )
                            print(
                                "Task paused, long press '"
                                + press_time["pause_key"]
                                + "' to continue..."
                            )
                            event.clear()
                        press_time["duration"] = time.time()
                        press_time["is_pressed"] = False
                    # print("按下p键时间：", press_time["duration"])
        except:
            pass

    return on_press


def on_release_creator(event, press_time):
    def on_release(key):
        try:
            # duration = time.time() - press_time["duration"]
            # # print("松开p键时间：", time.time(), "Duration: ", duration)
            # if duration > 2.5 and key.char == 'p':
            #     if event._flag == False:
            #         print("任务执行中，按p键暂停执行。")
            #         print("Task is running, press 'p' to pause.")
            #         # 设置Event的值为True，使得线程b可以继续执行
            #         event.set()
            #     else:
            #         # 设置Event的值为False，使得线程b暂停执行
            #         print("任务已暂停，按p键继续执行...")
            #         print("Task paused, press 'p' to continue...")
            #         event.clear()
            #     press_time["duration"] = time.time()
            press_time["is_pressed"] = False
        except:
            pass

    return on_release


# def check_pause(key, event):
#     while True:
#         if keyboard.is_pressed(key):  # 按下p键，暂停程序
#             if event._flag == False:
#                 print("任务执行中，长按p键暂停执行。")
#                 print("Task is running, long press 'p' to pause.")
#                 # 设置Event的值为True，使得线程b可以继续执行
#                 event.set()
#             else:
#                 # 设置Event的值为False，使得线程b暂停执行
#                 print("任务已暂停，长按p键继续执行...")
#                 print("Task paused, press 'p' to continue...")
#                 event.clear()
#         time.sleep(1)  # 每秒检查一次


def detect_optimizable(param, ignoreWaitElement=True, waitElement=""):
    try:
        splitLine = param["splitLine"]
    except:
        param["splitLine"] = 0
    if (
        param["beforeJS"] == ""
        and param["afterJS"] == ""
        and param["contentType"] <= 1
        and param["splitLine"] == 0
    ):
        if param["nodeType"] <= 2:
            if ignoreWaitElement or waitElement == "":
                return True
            else:
                return False
        elif param["nodeType"] == 4:  # 如果是图片
            if param["downloadPic"]:
                return False
            else:
                return True
    else:
        return False


def download_image(browser, url, save_directory, element=None):
    # 定义浏览器头信息
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36"
    }
    if url.startswith("data:image"):
        base64_data = url.split(",")[1]
        image_data = base64.b64decode(base64_data)
        # 提取文件名
        file_name = str(uuid.uuid4()) + ".png"
        # 构建保存路径
        save_path = os.path.join(save_directory, file_name)
        # 保存图片到本地
        with open(save_path, "wb") as file:
            file.write(image_data)
        browser.print_and_log("图片已成功下载到:", save_path)
        browser.print_and_log(
            "The image has been successfully downloaded to:", save_path
        )
    elif is_valid_url(url):
        try:
            # 提取文件名
            file_name = url.split("/")[-1].split("?")[0]

            # 生成唯一的新文件名
            new_file_name = file_name + "_" + str(uuid.uuid4()) + "_" + file_name

            # 构建保存路径
            save_path = os.path.join(save_directory, new_file_name)
            # 发送 GET 请求获取图片数据，加载浏览器的cookies
            s = requests.session()
            cookies = browser.browser.get_cookies()
            for cookie in cookies:
                s.cookies.set(cookie["name"], cookie["value"])
            response = s.get(url, headers=headers)
            # 检查响应状态码是否为成功状态
            if response.status_code == requests.codes.ok:
                # 保存图片到本地
                with open(save_path, "wb") as file:
                    file.write(response.content)
                browser.print_and_log("图片已成功下载到:", save_path)
                browser.print_and_log(
                    "The image has been successfully downloaded to:", save_path
                )
            else:
                # browser.print_and_log(f"直接下载图片失败，状态码为:{response.status_code}，尝试使用Selenium下载图片...")
                # browser.print_and_log(
                # f"Failed to download image directly, status code is: {response.status_code}, try to download image using Selenium...")
                JS = (
                    "var xhr = new XMLHttpRequest(); xhr.open('GET', '"
                    + url
                    + "', true); xhr.responseType = 'blob'; xhr.onload = function() {var reader = new FileReader(); reader.readAsDataURL(xhr.response); reader.onloadend = function() { var base64data = reader.result;}}; xhr.send();"
                    ""
                )
                base64data = browser.browser.execute_script(JS)
                if base64data:
                    image_data = base64data.b64decode(base64data.split(",")[1])
                    with open(save_path, "wb") as file:
                        file.write(image_data)
                    browser.print_and_log("图片已成功下载到:", save_path)
                    browser.print_and_log(
                        "The image has been successfully downloaded to:", save_path
                    )
                else:
                    browser.print_and_log(
                        "下载图片失败，只能使用元素截图功能下载图片。"
                    )
                    browser.print_and_log(
                        "Failed to download image, can only download image using element screenshot function."
                    )
                    # 使用元素截图功能下载图片
                    try:
                        element.screenshot(save_path)
                        browser.print_and_log("图片截图已保存到:", save_path)
                        browser.print_and_log(
                            "The image screenshot has been saved to:", save_path
                        )
                    except Exception as e:
                        browser.print_and_log(
                            "下载图片失败|Error downloading image: ", e
                        )
        except Exception as e:
            browser.print_and_log("下载图片失败|Error downloading image: ", e)
    else:
        browser.print_and_log("下载图片失败，请检查此图片链接是否有效:", url)
        browser.print_and_log(
            "Failed to download image, please check if this image link is valid:", url
        )


def get_output_code(output):
    try:
        if output.find("rue") != -1:  # 如果返回值中包含true
            code = 1
        else:
            code = int(output)
    except:
        code = 0
    return code


# 判断字段是否为空


def isnotnull(s):
    return len(s) != 0


def new_line(outputParameters, maxViewLength, record):
    line = []
    i = 0
    for value in outputParameters.values():
        line.append(value)
        if record[i]:
            print(value[:maxViewLength], " ", end="")
        i += 1
    print("")
    return line


def write_to_csv(file_name, data, record):
    with open(file_name, "a", encoding="utf-8-sig", newline="") as f:
        f_csv = csv.writer(f)
        for line in data:
            to_write = []
            for i in range(len(line)):
                if record[i]:
                    to_write.append(line[i])
            f_csv.writerow(to_write)
        f.close()


def replace_field_values(orginal_text, outputParameters, browser=None):
    pattern = r'Field\["([^"]+)"\]'
    try:
        replaced_text = re.sub(
            pattern,
            lambda match: outputParameters.get(match.group(1), ""),
            orginal_text,
        )
        if re.search(r"eval\(", replaced_text, re.IGNORECASE):  # 如果返回值中包含EVAL
            replaced_text = replaced_text.replace("self.", "browser.")
            pattern = re.compile(r'(?i)eval\("(.+?)"\)')
            # 循环替换所有匹配到的eval语句
            while True:
                match = pattern.search(replaced_text)
                if not match:
                    break
                # 执行eval并将其结果转换为字符串形式
                eval_replaced_text = str(eval(match.group(1)))
                # 替换eval语句
                replaced_text = replaced_text.replace(
                    match.group(0), eval_replaced_text
                )
        if re.search(r"JS\(", replaced_text, re.IGNORECASE):  # 如果返回值中包含JS
            replaced_text = replaced_text.replace("self.", "browser.")
            pattern = re.compile(r'(?i)JS\("(.+?)"\)')
            # 循环替换所有匹配到的JS语句
            while True:
                match = pattern.search(replaced_text)
                if not match:
                    break
                # 执行JS并将其结果转换为字符串形式
                JS_replaced_text = str(browser.browser.execute_script(match.group(1)))
                # 替换JS语句
                replaced_text = replaced_text.replace(match.group(0), JS_replaced_text)
    except Exception as e:
        print(
            "eval替换失败，请检查eval语句是否正确。| Failed to replace eval, please check if the eval statement is correct."
        )
        print(e)
        replaced_text = orginal_text
    return replaced_text


def readCode(code):
    if code.startswith("outside:"):
        file_name = os.path.join(os.path.abspath("./"), code[8:])
        with open(file_name, "r", encoding="utf-8-sig") as file_obj:
            code = file_obj.read()
    return code


def write_to_json(file_name, data, types, record, keys):
    keys = list(keys)
    # Prepare empty list for data
    data_to_write = []
    # Tranform data and append to list
    for line in data:
        to_write = {}
        for i in range(len(line)):
            if types[i] == "int" or types[i] == "bigInt":
                try:
                    line[i] = int(line[i])
                except:
                    line[i] = 0
            elif types[i] == "double":
                try:
                    line[i] = float(line[i])
                except:
                    line[i] = 0.0
            if record[i]:
                to_write.update({keys[i]: line[i]})
        data_to_write.append(to_write)

    try:
        # read data from JSON
        with open(file_name, "r", encoding="utf-8") as f:
            json_data = json.load(f)
    except:
        json_data = []

    json_data.extend(data_to_write)

    # write data to JSON
    with open(file_name, "w", encoding="utf-8") as f:
        json.dump(json_data, f, ensure_ascii=False)


def write_to_excel(file_name, data, types, record):
    # 首先，检查文件是否存在来决定是否处理第一行
    # first = not os.path.exists(file_name)

    # # 准备新数据
    # new_data = pd.DataFrame(data)

    # # 如果不是第一行（即文件已存在），对数据应用类型转换
    # if not first:
    #     for i, col_type in enumerate(types):
    #         if col_type == "int" or col_type == "bigInt":
    #             try:
    #                 new_data[i] = pd.to_numeric(new_data[i], errors='coerce').astype(int)
    #             except:
    #                 new_data[i] = pd.to_numeric("0", errors='coerce').astype(int)
    #         elif col_type == "double":
    #             try:
    #                 new_data[i] = pd.to_numeric(new_data[i], errors='coerce')(0.0)
    #             except:
    #                 new_data[i] = pd.to_numeric("0.0", errors='coerce').astype(float)
    # # 根据 record 筛选列
    # new_data = new_data.loc[:, record]

    # # 如果文件存在，则读取现有数据并追加新数据
    # if first:
    #     combined_data = new_data
    # else:
    #     # 使用 Pandas 读取现有数据
    #     existing_data = pd.read_excel(file_name)
    #     # 合并现有数据与新数据
    #     combined_data = pd.concat([existing_data, new_data], ignore_index=True)

    # # 将合并后的数据写入 Excel
    # combined_data.to_excel(file_name, index=False, engine='openpyxl')

    # existing_data = []
    # first = True
    # # 检查文件是否存在
    # if os.path.exists(file_name):
    #     # 使用 openpyxl 读取现有数据
    #     workbook = load_workbook(file_name, read_only=True)
    #     sheet = workbook.active
    #     # 读取已有行数
    #     num_rows = sheet.max_row
    #     if num_rows > 5000:
    #         print("Excel文件中的数据行数超过5000行，过多的行数将会导致追加模式写入数据速度变慢，建议更换为CSV文件或MySQL数据库存储数据。正在读取数据，请稍等...")
    #         print("The number of rows in the Excel file exceeds 5000, too many rows will cause the speed of writing data in append mode to slow down, it is recommended to replace it with CSV file or MySQL database to store data. Reading data, please wait...")
    #     # existing_data = [[sheet.cell(row=i, column=j).value for j in range(1, sheet.max_column + 1)] for i in range(1, sheet.max_row + 1)]
    #     for i in range(1, sheet.max_row + 1):
    #         row_data = []
    #         if num_rows > 5000 and i % 500 == 0:
    #             print(f"正在读取第{i}/{num_rows}行的数据...")
    #             print(f"Reading data of row {i}/{num_rows}...")
    #         for j in range(1, sheet.max_column + 1):
    #             cell = sheet.cell(row=i, column=j).value
    #             if cell is None:
    #                 cell = ""
    #             row_data.append(cell)
    #         existing_data.append(row_data)
    #     first = False  # 如果文件存在，首行不再是标题行

    # # 使用 xlsxwriter 创建新文件
    # workbook = xlsxwriter.Workbook(file_name)
    # worksheet = workbook.add_worksheet()

    # # 写入现有数据
    # for row_num, row_data in enumerate(existing_data):
    #     for col_num, cell in enumerate(row_data):
    #         worksheet.write(row_num, col_num, cell)

    # # 写入新数据
    # row = len(existing_data)
    # for line in data:
    #     to_write = []
    #     for i in range(len(line)):
    #         value = line[i]
    #         if not first:  # 如果不是第一行，需要转换数据类型
    #             if types[i] == "int" or types[i] == "bigInt":
    #                 try:
    #                     value = int(value)
    #                 except ValueError:
    #                     value = 0
    #             elif types[i] == "double":
    #                 try:
    #                     value = float(value)
    #                 except ValueError:
    #                     value = 0.0
    #         if record[i]:
    #             to_write.append(value)
    #     first = False  # 更新 first 以跳过数据类型转换
    #     for col, item in enumerate(to_write):
    #         worksheet.write(row, col, item)
    #     row += 1

    # # 关闭工作簿
    # workbook.close()

    first = False
    if os.path.exists(file_name):
        # 加载现有的工作簿
        wb = load_workbook(file_name)
        # 行数读取
        num_rows = wb.active.max_row
        if num_rows > 1000:
            print(
                "Excel文件中的数据行数已超过1000行，过多的行数将会导致追加模式写入数据速度变慢，建议增大任务保存对话框中的“每采集多少条数据保存一次”选项的值以提升采集速度，或者更换为CSV文件或MySQL数据库存储数据。正在读取数据，请稍等..."
            )
            print(
                "The number of rows in the Excel file already exceeds 1000, too many rows will cause the speed of writing data in append mode to slow down, it is recommended to increase the value of the 'Save every how many data' option in the task save dialog to improve the collection speed, or replace it with CSV file or MySQL database to store data. Reading data, please wait..."
            )
        ws = wb.active
        if num_rows > 1000:
            print("读取数据完成，正在追加数据...")
            print("Reading data completed, appending data...")
    else:
        # 创建新的工作簿和工作表
        wb = Workbook()
        ws = wb.active
        first = True
    # 追加数据到工作表
    for line in data:
        if not first:  # 如果不是第一行，需要转换数据类型
            for i in range(len(line)):
                if types[i] == "int" or types[i] == "bigInt":
                    try:
                        line[i] = int(line[i])
                    except:
                        line[i] = 0
                elif types[i] == "double":
                    try:
                        line[i] = float(line[i])
                    except:
                        line[i] = 0.0
        else:
            first = False
        to_write = []
        for i in range(len(line)):
            if record[i]:
                to_write.append(line[i])
        try:
            ws.append(to_write)
        except:
            print("写入Excel文件失败，请检查数据类型是否正确。")
            print(
                "Failed to write to Excel file, please check if the data type is correct."
            )
    # 保存工作簿
    try:
        wb.save(file_name)
    except:
        print("保存Excel文件失败，请检查文件是否被其他程序打开。")
        print(
            "Failed to save Excel file, please check if the file is opened by other programs."
        )


class Time:
    def __init__(self, type1=""):
        self.t = int(round(time.time() * 1000))
        self.type = type1

    def end(self):
        at = int(round(time.time() * 1000))
        print("Time used for", self.type, ":", at - self.t, "ms")


class myMySQL:
    def __init__(self, config_file="mysql_config.json"):
        # 读取配置文件
        try:
            if sys.platform == "darwin":
                if config_file.find("./") >= 0:
                    config_file = config_file.replace("./", "")
                config_file = os.path.expanduser(
                    "~/Library/Application Support/EasySpider/" + config_file
                )
            print("MySQL config file path: ", config_file)
            with open(config_file, "r") as f:
                config = json.load(f)
                self.host = config["host"]
                self.port = config["port"]
                self.username = config["username"]
                self.password = config["password"]
                self.db = config["database"]
        except Exception as e:
            print(
                "读取配置文件失败，请检查配置文件："
                + config_file
                + "是否存在，或配置信息是否有误。"
            )
            print(
                "Failed to read configuration file, please check if the configuration file: "
                + config_file
                + " exists, or if the configuration information is incorrect."
            )
            print(e)
        self.connect()

    def connect(self):
        try:
            self.conn = pymysql.connect(
                host=self.host,
                port=self.port,
                user=self.username,
                passwd=self.password,
                db=self.db,
            )
            print("成功连接到数据库。")
            print("Successfully connected to the database.")
        except:
            print("连接数据库失败，请检查配置文件是否正确。")
            print(
                "Failed to connect to the database, please check if the configuration file is correct."
            )
            sys.exit()

    def create_table(self, table_name, parameters, remove_if_exists=False):
        self.table_name = table_name
        self.field_sql = "("
        self.cursor = self.conn.cursor()
        # 检查表是否存在
        self.cursor.execute(f"SHOW TABLES LIKE '{table_name}'")
        result = self.cursor.fetchone()
        # 如果表存在，删除它
        if result and remove_if_exists:
            self.cursor.execute(f"DROP TABLE {table_name}")
            result = None
            print(f"数据表 {table_name} 已存在，已删除。")

        sql = "CREATE TABLE " + table_name + " (_id INT AUTO_INCREMENT PRIMARY KEY, "
        for item in parameters:
            try:
                recordASField = item["recordASField"]
            except:
                item["recordASField"] = True
            if item["recordASField"]:
                name = item["name"]
                if item["type"] == "int":
                    sql += f"{name} INT, "
                elif item["type"] == "double":
                    sql += f"{name} DOUBLE, "
                elif item["type"] == "text":
                    sql += f"{name} TEXT, "
                elif item["type"] == "mediumText":
                    sql += f"{name} MEDIUMTEXT, "
                elif item["type"] == "longText":
                    sql += f"{name} LONGTEXT, "
                elif item["type"] == "datetime":
                    sql += f"{name} DATETIME, "
                elif item["type"] == "date":
                    sql += f"{name} DATE, "
                elif item["type"] == "time":
                    sql += f"{name} TIME, "
                elif item["type"] == "varchar":
                    sql += f"{name} VARCHAR(255), "
                elif item["type"] == "bigInt":
                    sql += f"{name} BIGINT, "
                self.field_sql += f"{name}, "
        # 移除最后的逗号并添加闭合的括号
        sql = sql.rstrip(", ") + ")"
        self.field_sql = self.field_sql.rstrip(", ") + ")"

        # 如果表不存在，创建它
        if not result:
            # 执行SQL命令
            self.cursor.execute(sql)
        else:
            print(f"数据表 {table_name} 已存在")
            print(f"The data table {table_name} already exists.")
        self.cursor.close()

    def write_to_mysql(self, OUTPUT, record, types):
        # 创建一个游标对象
        self.cursor = self.conn.cursor()

        for line in OUTPUT:
            for i in range(len(line)):
                if types[i] == "int" or types[i] == "bigInt":
                    try:
                        line[i] = int(line[i])
                    except Exception as e:
                        print(e)
                        line[i] = 0
                elif types[i] == "double":
                    try:
                        line[i] = float(line[i])
                    except Exception as e:
                        print(e)
                        line[i] = 0.0
                elif types[i] == "datetime":
                    try:
                        line[i] = datetime.datetime.strptime(
                            line[i], "%Y-%m-%d %H:%M:%S"
                        )
                    except Exception as e:
                        print(e)
                        line[i] = datetime.datetime.strptime(
                            "1970-01-01 00:00:00", "%Y-%m-%d %H:%M:%S"
                        )
                elif types[i] == "date":
                    try:
                        line[i] = datetime.datetime.strptime(line[i], "%Y-%m-%d")
                    except Exception as e:
                        print(e)
                        line[i] = datetime.datetime.strptime("1970-01-01", "%Y-%m-%d")
                elif types[i] == "time":
                    try:
                        line[i] = datetime.datetime.strptime(line[i], "%H:%M:%S")
                    except Exception as e:
                        print(e)
                        line[i] = datetime.datetime.strptime("00:00:00", "%H:%M:%S")
            to_write = []
            for i in range(len(line)):
                if record[i]:
                    to_write.append(line[i])
            # 构造插入数据的 SQL 语句, IGNORE表示如果主键重复则忽略
            sql = f"INSERT IGNORE INTO {self.table_name} {self.field_sql} VALUES ("
            for _ in to_write:
                sql += "%s, "
            # 移除最后的逗号并添加闭合的括号
            sql = sql.rstrip(", ") + ")"
            # 执行 SQL 语句
            try:
                self.cursor.execute(sql, to_write)
            except pymysql.OperationalError as e:
                print("Error:", e)
                print("Try to reconnect to the database...")
                self.connect()
                self.cursor = self.conn.cursor()  # 重新创建游标对象
                self.cursor.execute(sql, to_write)  # 重新执行SQL语句
                # self.write_to_mysql(OUTPUT, record, types)
            except Exception as e:
                print("Error:", e)
                print("Error SQL:", sql, to_write)
                print(
                    "插入数据库错误，请查看以上的错误提示，然后检查数据的类型是否正确，是否文本过长（超过一万的文本类型要设置为大文本）。"
                )
                print(
                    "Inserting database error, please check the above error, and then check whether the data type is correct, whether the text is too long (text type over 10,000 should be set to large text)."
                )
                print(
                    "重新执行任务时，请删除数据库中的数据表"
                    + self.table_name
                    + "，然后再次运行程序。"
                )
                print(
                    "When re-executing the task, please delete the data table "
                    + self.table_name
                    + " in the database, and then run the program again."
                )

        # 提交到数据库执行
        self.conn.commit()

        # 关闭游标和连接
        self.cursor.close()

    def remove_duplicate_data(self):
        self.cursor = self.conn.cursor()
        # 删除重复数据
        fields = self.field_sql.replace("(", "").replace(")", "")
        sql = (
            f"CREATE TABLE {self.table_name}_temp AS "
            + "SELECT MIN(_id) AS _id, "
            + fields
            + f" FROM {self.table_name} GROUP BY "
            + fields
            + ";"
        )
        self.cursor.execute(sql)
        sql = f"DELETE FROM {self.table_name};"
        self.cursor.execute(sql)
        sql = f"INSERT INTO {self.table_name} SELECT * FROM {self.table_name}_temp;"
        self.cursor.execute(sql)
        sql = f"DROP TABLE {self.table_name}_temp;"
        self.cursor.execute(sql)
        # 提交到数据库执行
        self.conn.commit()
        # 关闭游标和连接
        self.cursor.close()

    def close(self):
        try:
            self.conn.close()
            print("成功关闭数据库。")
            print("Successfully closed the database.")
        except:
            print("关闭数据库失败。")
            print("Failed to close the database.")

    def __del__(self):
        self.close()
